# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import zipfile
from dataclasses import dataclass
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import (SCHEMA_TO_FEATURES, TASK_TO_SCHEMA,
                                      Licenses, Tasks)

_CITATION = """\
@article{nllb2022,
  author    = {NLLB Team, Marta R. Costa-jussà, James Cross, Onur Çelebi, Maha Elbayad, Kenneth Heafield, Kevin Heffernan, Elahe
  Kalbassi,  Janice Lam, Daniel Licht, Jean Maillard, Anna Sun, Skyler Wang, Guillaume Wenzek, Al Youngblood, Bapi Akula, Loic
  Barrault, Gabriel Mejia Gonzalez, Prangthip Hansanti, John Hoffman, Semarley Jarrett, Kaushik Ram Sadagopan, Dirk Rowe, Shannon
  Spruit, Chau Tran, Pierre Andrews, Necip Fazil Ayan, Shruti Bhosale, Sergey Edunov, Angela Fan, Cynthia Gao, Vedanuj Goswami,
  Francisco Guzmán, Philipp Koehn, Alexandre Mourachko, Christophe Ropers, Safiyyah Saleem, Holger Schwenk, Jeff Wang},
  title     = {No Language Left Behind: Scaling Human-Centered Machine Translation},
  year      = {2022}
}
@inproceedings{,
  title={The FLORES-101  Evaluation Benchmark for Low-Resource and Multilingual Machine Translation},
  author={Goyal, Naman and Gao, Cynthia and Chaudhary, Vishrav and Chen, Peng-Jen and Wenzek, Guillaume and Ju, Da and Krishnan, Sanjana and Ranzato, Marc'Aurelio and Guzm\'{a}n, Francisco and Fan, Angela},
  year={2021}
}
@inproceedings{,
  title={Two New Evaluation Datasets for Low-Resource Machine Translation: Nepali-English and Sinhala-English},
  author={Guzm\'{a}n, Francisco and Chen, Peng-Jen and Ott, Myle and Pino, Juan and Lample, Guillaume and Koehn, Philipp and Chaudhary, Vishrav and Ranzato, Marc'Aurelio},
  journal={arXiv preprint arXiv:1902.01382},
  year={2019}
}
"""

_DATASETNAME = "flores200"

_DESCRIPTION = """\
The creation of FLORES-200 doubles the existing language coverage of FLORES-101.
Given the nature of the new languages, which have less standardization and require
more specialized professional translations, the verification process became more complex.
This required modifications to the translation workflow. FLORES-200 has several languages
which were not translated from English. Specifically, several languages were translated
from Spanish, French, Russian and Modern Standard Arabic. Moreover, FLORES-200 also
includes two script alternatives for four languages. FLORES-200 consists of translations
from 842 distinct web articles, totaling 3001 sentences. These sentences are divided
into three splits: dev, devtest, and test (hidden). On average, sentences are approximately
21 words long.
"""

_HOMEPAGE = "https://github.com/facebookresearch/flores"

_LANGUAGES = [
    "ace",
    "ban",
    "bjn",
    "bug",
    "ceb",
    "ilo",
    "ind",
    "jav",
    "kac",
    "khm",
    "lao",
    "lus",
    "min",
    "mya",
    "pag",
    "shn",
    "sun",
    "tgl",
    "tha",
    "vie",
    "war",
    "zsm",
]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)

_LANGUAGE_NAMES = [
    "ace_Arab",
    "ace_Latn",
    "acm_Arab",
    "acq_Arab",
    "aeb_Arab",
    "afr_Latn",
    "ajp_Arab",
    "aka_Latn",
    "als_Latn",
    "amh_Ethi",
    "apc_Arab",
    "arb_Arab",
    "arb_Latn",
    "ars_Arab",
    "ary_Arab",
    "arz_Arab",
    "asm_Beng",
    "ast_Latn",
    "awa_Deva",
    "ayr_Latn",
    "azb_Arab",
    "azj_Latn",
    "bak_Cyrl",
    "bam_Latn",
    "ban_Latn",
    "bel_Cyrl",
    "bem_Latn",
    "ben_Beng",
    "bho_Deva",
    "bjn_Arab",
    "bjn_Latn",
    "bod_Tibt",
    "bos_Latn",
    "bug_Latn",
    "bul_Cyrl",
    "cat_Latn",
    "ceb_Latn",
    "ces_Latn",
    "cjk_Latn",
    "ckb_Arab",
    "cmn_Hans",
    "cmn_Hant",
    "crh_Latn",
    "cym_Latn",
    "dan_Latn",
    "deu_Latn",
    "dik_Latn",
    "dyu_Latn",
    "dzo_Tibt",
    "ell_Grek",
    "eng_Latn",
    "epo_Latn",
    "est_Latn",
    "eus_Latn",
    "ewe_Latn",
    "fao_Latn",
    "fij_Latn",
    "fin_Latn",
    "fon_Latn",
    "fra_Latn",
    "fur_Latn",
    "fuv_Latn",
    "gla_Latn",
    "gle_Latn",
    "glg_Latn",
    "grn_Latn",
    "guj_Gujr",
    "hat_Latn",
    "hau_Latn",
    "heb_Hebr",
    "hin_Deva",
    "hne_Deva",
    "hrv_Latn",
    "hun_Latn",
    "hye_Armn",
    "ibo_Latn",
    "ilo_Latn",
    "ind_Latn",
    "isl_Latn",
    "ita_Latn",
    "jav_Latn",
    "jpn_Jpan",
    "kab_Latn",
    "kac_Latn",
    "kam_Latn",
    "kan_Knda",
    "kas_Arab",
    "kas_Deva",
    "kat_Geor",
    "knc_Arab",
    "knc_Latn",
    "kaz_Cyrl",
    "kbp_Latn",
    "kea_Latn",
    "khm_Khmr",
    "kik_Latn",
    "kin_Latn",
    "kir_Cyrl",
    "kmb_Latn",
    "kmr_Latn",
    "kon_Latn",
    "kor_Hang",
    "lao_Laoo",
    "lij_Latn",
    "fil_Latn",
    "lim_Latn",
    "lin_Latn",
    "lit_Latn",
    "lmo_Latn",
    "ltg_Latn",
    "ltz_Latn",
    "lua_Latn",
    "lug_Latn",
    "luo_Latn",
    "lus_Latn",
    "lvs_Latn",
    "mag_Deva",
    "mai_Deva",
    "mal_Mlym",
    "mar_Deva",
    "min_Arab",
    "min_Latn",
    "mkd_Cyrl",
    "plt_Latn",
    "mlt_Latn",
    "mni_Beng",
    "khk_Cyrl",
    "mos_Latn",
    "mri_Latn",
    "mya_Mymr",
    "nld_Latn",
    "nno_Latn",
    "nob_Latn",
    "npi_Deva",
    "nqo_Nkoo",
    "nso_Latn",
    "nus_Latn",
    "nya_Latn",
    "oci_Latn",
    "gaz_Latn",
    "ory_Orya",
    "pag_Latn",
    "pan_Guru",
    "pap_Latn",
    "pes_Arab",
    "pol_Latn",
    "por_Latn",
    "prs_Arab",
    "pbt_Arab",
    "quy_Latn",
    "ron_Latn",
    "run_Latn",
    "rus_Cyrl",
    "sag_Latn",
    "san_Deva",
    "sat_Olck",
    "scn_Latn",
    "shn_Mymr",
    "sin_Sinh",
    "slk_Latn",
    "slv_Latn",
    "smo_Latn",
    "sna_Latn",
    "snd_Arab",
    "som_Latn",
    "sot_Latn",
    "spa_Latn",
    "srd_Latn",
    "srp_Cyrl",
    "ssw_Latn",
    "sun_Latn",
    "swe_Latn",
    "swh_Latn",
    "szl_Latn",
    "tam_Taml",
    "tat_Cyrl",
    "tel_Telu",
    "tgk_Cyrl",
    "tha_Thai",
    "tir_Ethi",
    "taq_Latn",
    "taq_Tfng",
    "tpi_Latn",
    "tsn_Latn",
    "tso_Latn",
    "tuk_Latn",
    "tum_Latn",
    "tur_Latn",
    "twi_Latn",
    "uig_Arab",
    "ukr_Cyrl",
    "umb_Latn",
    "urd_Arab",
    "uzn_Latn",
    "vec_Latn",
    "vie_Latn",
    "war_Latn",
    "wol_Latn",
    "xho_Latn",
    "ydd_Hebr",
    "yor_Latn",
    "yue_Hant",
    "zgh_Tfng",
    "zsm_Latn",
    "zul_Latn",
]

_LICENSE = Licenses.CC_BY_NC_4_0.value

_LOCAL = False

_URLS = {
    _DATASETNAME: "https://github.com/openlanguagedata/flores/releases/download/v2.0-alpha.2/floresp-v2.0-alpha.2.zip",
}

_SPLITS = ["dev", "devtest"]

_SENTENCES_PATHS = {lang: {split: os.path.join("floresp-v2.0-alpha.2", split, f"{split}.{lang}") for split in _SPLITS} for lang in _LANGUAGE_NAMES}

_METADATA_PATHS = {split: os.path.join("floresp-v2.0-alpha.2", f"metadata_{split}.tsv") for split in _SPLITS}

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]
_SUPPORTED_SCHEMA_STRINGS = [f"seacrowd_{str(TASK_TO_SCHEMA[task]).lower()}" for task in _SUPPORTED_TASKS]

_SCHEMAS = [str(TASK_TO_SCHEMA[task]) for task in _SUPPORTED_TASKS]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


@dataclass
class Flores200SeacrowdConfig(SEACrowdConfig):
    """BuilderConfig for Nusantara."""

    first_language_name: str = None
    second_language_name: str = None


class Flores200(datasets.GeneratorBasedBuilder):
    """
    The creation of FLORES-200 doubles the existing language coverage of FLORES-101.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = []

    for first_lang_name in _LANGUAGE_NAMES:
        for second_lang_name in _LANGUAGE_NAMES:
            if first_lang_name == second_lang_name or ((first_lang_name.split("_")[0] not in _LANGUAGES) and (second_lang_name.split("_")[0] not in _LANGUAGES)):
                continue

            subset_id = f"{_DATASETNAME}_{first_lang_name}_{second_lang_name}"

            BUILDER_CONFIGS.append(
                Flores200SeacrowdConfig(
                    name=f"{subset_id}_source",
                    version=SOURCE_VERSION,
                    description=f"{_DATASETNAME} source schema",
                    schema="source",
                    subset_id=subset_id,
                    first_language_name=first_lang_name,
                    second_language_name=second_lang_name,
                )
            )

            seacrowd_schema_config: list[SEACrowdConfig] = []

            for seacrowd_schema in _SUPPORTED_SCHEMA_STRINGS:

                seacrowd_schema_config.append(
                    Flores200SeacrowdConfig(
                        name=f"{subset_id}_{seacrowd_schema}",
                        version=SEACROWD_VERSION,
                        description=f"{_DATASETNAME} {seacrowd_schema} schema",
                        schema=f"{seacrowd_schema}",
                        subset_id=subset_id,
                        first_language_name=first_lang_name,
                        second_language_name=second_lang_name,
                    )
                )

            BUILDER_CONFIGS.extend(seacrowd_schema_config)

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_{_LANGUAGE_NAMES[0]}_{_LANGUAGE_NAMES[1]}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("int32"),
                    "URL": datasets.Value("string"),
                    "domain": datasets.Value("string"),
                    "topic": datasets.Value("string"),
                    "has_image": datasets.Value("int32"),
                    "has_hyperlink": datasets.Value("int32"),
                }
            )

            features[self.config.first_language_name] = datasets.Value("string")
            features[self.config.second_language_name] = datasets.Value("string")

        else:
            schema = str(self.config.schema).lstrip(f"{_DATASETNAME}_seacrowd_").upper()

            if schema in _SCHEMAS:
                features = SCHEMA_TO_FEATURES[schema]

            else:
                raise ValueError(f"Invalid config: {self.config.name}")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        dl_dir = dl_manager.download(_URLS[_DATASETNAME])

        base_dir = os.path.join(os.path.dirname(dl_dir), "flores200extracted")

        password = "multilingual machine translation"

        with zipfile.ZipFile(dl_dir, "r") as zip_ref:
            # Set the password to extract the contents
            zip_ref.setpassword(bytes(password, "utf-8"))

            # Extract all contents to the specified directory
            zip_ref.extractall(base_dir)

        return [
            datasets.SplitGenerator(
                name=split,
                gen_kwargs={
                    "first_sentence_path": os.path.join(base_dir, _SENTENCES_PATHS[self.config.first_language_name][split]),
                    "second_sentence_path": os.path.join(base_dir, _SENTENCES_PATHS[self.config.second_language_name][split]),
                    "metadata_path": os.path.join(base_dir, _METADATA_PATHS[split]),
                },
            )
            for split in _SPLITS
        ]

    def _generate_examples(self, first_sentence_path: str, second_sentence_path: str, metadata_path: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""

        sentences = {}
        langs = [self.config.first_language_name, self.config.second_language_name]

        for path, lang in zip([first_sentence_path, second_sentence_path], langs):
            with open(path, "r") as sent_file:
                sentences[lang] = [line.strip() for line in sent_file.readlines()]

        with open(metadata_path, "r") as metadata_file:
            metadata_lines = [line.strip() for line in metadata_file.readlines()[1:]]

        if self.config.schema == "source":
            for id_, metadata in enumerate(metadata_lines):
                metadata = metadata.split("\t")
                yield id_, {
                    **{"id": id_ + 1, "URL": metadata[0], "domain": metadata[1], "topic": metadata[2], "has_image": 1 if metadata == "yes" else 0, "has_hyperlink": 1 if metadata == "yes" else 0},
                    **{f"{lang}": sentences[lang][id_] for lang in langs},
                }

        elif self.config.schema == f"seacrowd_{str(TASK_TO_SCHEMA[Tasks.MACHINE_TRANSLATION]).lower()}":
            for id_, _ in enumerate(metadata_lines):
                yield id_, {
                    "id": id_ + 1,
                    "text_1": sentences[self.config.first_language_name][id_],
                    "text_2": sentences[self.config.second_language_name][id_],
                    "text_1_name": self.config.first_language_name,
                    "text_2_name": self.config.second_language_name,
                }

        else:
            raise ValueError(f"Invalid config: {self.config.name}")



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
from datasets.download.download_manager import DownloadManager

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@article{tatoeba,
    title     = {Massively Multilingual Sentence Embeddings for Zero-Shot Cross-Lingual Transfer and Beyond},
    author    = {Mikel, Artetxe and Holger, Schwenk,},
    journal   = {arXiv:1812.10464v2},
    year      = {2018}
}
"""

_LOCAL = False
_LANGUAGES = ["ind", "vie", "tgl", "jav", "tha"]
_DATASETNAME = "tatoeba"
_DESCRIPTION = """\
This dataset is a subset of the Tatoeba corpus containing language pairs for Indonesian, Vietnamese, Tagalog, Javanese, and Thai.
The original dataset description can be found below:

This data is extracted from the Tatoeba corpus, dated Saturday 2018/11/17.
For each languages, we have selected 1000 English sentences and their translations, if available. Please check
this paper for a description of the languages, their families and scripts as well as baseline results.
Please note that the English sentences are not identical for all language pairs. This means that the results are
not directly comparable across languages. In particular, the sentences tend to have less variety for several
low-resource languages, e.g. "Tom needed water", "Tom needs water", "Tom is getting water", ...
"""

_HOMEPAGE = "https://github.com/facebookresearch/LASER/blob/main/data/tatoeba/v1/README.md"
_LICENSE = Licenses.APACHE_2_0.value
_URL = "https://github.com/facebookresearch/LASER/raw/main/data/tatoeba/v1/"

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class TatoebaDataset(datasets.GeneratorBasedBuilder):
    """Tatoeba subset for Indonesian, Vietnamese, Tagalog, Javanese, and Thai."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "t2t"

    # Add configurations for loading a dataset per language.
    dataset_names = sorted([f"tatoeba_{lang}" for lang in _LANGUAGES])
    BUILDER_CONFIGS = []
    for name in dataset_names:
        source_config = SEACrowdConfig(
            name=f"{name}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=name,
        )
        BUILDER_CONFIGS.append(source_config)
        seacrowd_config = SEACrowdConfig(
            name=f"{name}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=name,
        )
        BUILDER_CONFIGS.append(seacrowd_config)

    # Add configuration that allows loading all datasets at once.
    BUILDER_CONFIGS.extend(
        [
            # tatoeba_source
            SEACrowdConfig(
                name=f"{_DATASETNAME}_source",
                version=SOURCE_VERSION,
                description=f"{_DATASETNAME} source schema (all)",
                schema="source",
                subset_id=_DATASETNAME,
            ),
            # tatoeba_seacrowd_t2t
            SEACrowdConfig(
                name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
                version=SEACROWD_VERSION,
                description=f"{_DATASETNAME} SEACrowd schema (all)",
                schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
                subset_id=_DATASETNAME,
            ),
        ]
    )

    # Choose first language as default
    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "source_sentence": datasets.Value("string"),
                    "target_sentence": datasets.Value("string"),
                    "source_lang": datasets.Value("string"),
                    "target_lang": datasets.Value("string"),
                }
            )
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.text2text_features
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: DownloadManager) -> List[datasets.SplitGenerator]:
        """Return SplitGenerators."""
        languages = []
        tatoeba_source_data = []
        tatoeba_eng_data = []

        lang = self.config.name.split("_")[1]
        if lang in _LANGUAGES:
            # Load data per language
            tatoeba_source_data.append(dl_manager.download_and_extract(_URL + f"tatoeba.{lang}-eng.{lang}"))
            tatoeba_eng_data.append(dl_manager.download_and_extract(_URL + f"tatoeba.{lang}-eng.eng"))
            languages.append(lang)
        else:
            # Load examples from all languages at once
            # We just want to run this part when tatoeba_source / tatoeba_seacrowd_t2t was chosen.
            for lang in _LANGUAGES:
                tatoeba_source_data.append(dl_manager.download_and_extract(_URL + f"tatoeba.{lang}-eng.{lang}"))
                tatoeba_eng_data.append(dl_manager.download_and_extract(_URL + f"tatoeba.{lang}-eng.eng"))
                languages.append(lang)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepaths": (tatoeba_source_data, tatoeba_eng_data),
                    "split": "dev",
                    "languages": languages,
                },
            )
        ]

    def _generate_examples(self, filepaths: Tuple[List[Path], List[Path]], split: str, languages: List[str]) -> Tuple[int, Dict]:
        """Yield examples as (key, example) tuples"""
        source_files, target_files = filepaths
        source_sents = []
        target_sents = []
        source_langs = []

        for source_file, target_file, lang in zip(source_files, target_files, languages):
            with open(source_file, encoding="utf-8") as f1:
                for row in f1:
                    source_sents.append(row.strip())
                    source_langs.append(lang)
            with open(target_file, encoding="utf-8") as f2:
                for row in f2:
                    target_sents.append(row.strip())

        for idx, (source, target, lang) in enumerate(zip(source_sents, target_sents, source_langs)):
            if self.config.schema == "source":
                example = {
                    "source_sentence": source,
                    "target_sentence": target,
                    # The source_lang in the HuggingFace source seems incorrect
                    # I am overriding it with the actual language code.
                    "source_lang": lang,
                    "target_lang": "eng",
                }
            elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
                example = {
                    "id": str(idx),
                    "text_1": source,
                    "text_2": target,
                    # The source_lang in the HuggingFace source seems incorrect
                    # I am overriding it with the actual language code.
                    "text_1_name": lang,
                    "text_2_name": "eng",
                }
            yield idx, example



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This dataset contains audio recordings and phonetic transcriptions of word utterances for various low-resource SEA languages.
Each language has a directory of text and audio files, with the latter forming one data subset.
The dataset is prepared from the online UCLA phonetic dataset, which contains 7000 utterances across 100 low-resource languages, phonetically aligned using various automatic approaches, and manually fixed for misalignments.
"""
import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@inproceedings{li2021multilingual,
  title={Multilingual phonetic dataset for low resource speech recognition},
  author={Li, Xinjian and Mortensen, David R and Metze, Florian and Black, Alan W},
  booktitle={ICASSP 2021-2021 IEEE International Conference on Acoustics, Speech and Signal Processing (ICASSP)},
  pages={6958--6962},
  year={2021},
  organization={IEEE}
}
"""

_DATASETNAME = "ucla_phonetic"

_DESCRIPTION = """\
This dataset contains audio recordings and phonetic transcriptions of word utterances for various low-resource SEA languages.
Each language has a directory of text and audio files, with the latter forming one data subset.
The dataset is prepared from the online UCLA phonetic dataset, which contains 7000 utterances across 100 low-resource languages, phonetically aligned using various automatic approaches, and manually fixed for misalignments.
"""

_HOMEPAGE = "https://github.com/xinjli/ucla-phonetic-corpus"

_LANGUAGES = ["ace", "brv", "hil", "hni", "ilo", "khm", "mak", "mya", "pam"]

_LICENSE = Licenses.CC_BY_NC_SA_4_0.value

_LOCAL = False

_DATA_URL = "https://github.com/xinjli/ucla-phonetic-corpus/releases/download/v1.0/data.tar.gz"

_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


def seacrowd_config_constructor(lang, schema, version):
    if lang not in _LANGUAGES:
        raise ValueError(f"Invalid lang {lang}")

    if schema not in ["source", "seacrowd_sptext"]:
        raise ValueError(f"Invalid schema: {schema}")

    return SEACrowdConfig(
        name=f"ucla_phonetic_{lang}_{schema}",
        version=datasets.Version(version),
        description=f"UCLA Phonetic {schema} for {lang}",
        schema=schema,
        subset_id=f"{lang}_{schema}",
    )


class UCLAPhoneticDataset(datasets.GeneratorBasedBuilder):
    """This dataset contains audio recordings and phonetic transcriptions of word utterances for various low-resource SEA languages."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = (
        [
            SEACrowdConfig(
                name="ucla_phonetic_source",
                version=datasets.Version(_SOURCE_VERSION),
                description="UCLA Phonetic source for ace",
                schema="source",
                subset_id="ace_source",
            ),
            SEACrowdConfig(
                name="ucla_phonetic_seacrowd_sptext",
                version=datasets.Version(_SOURCE_VERSION),
                description="UCLA Phonetic seacrowd+sptext for ace",
                schema="seacrowd_sptext",
                subset_id="ace_seacrowd_sptext",
            ),
        ]
        + [seacrowd_config_constructor(lang, "source", _SOURCE_VERSION) for lang in _LANGUAGES]
        + [seacrowd_config_constructor(lang, "seacrowd_sptext", _SEACROWD_VERSION) for lang in _LANGUAGES]
    )

    DEFAULT_CONFIG_NAME = "ucla_phonetic_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features({"id": datasets.Value("string"), "text": datasets.Value("string"), "audio": datasets.Audio(sampling_rate=16_000)})
        elif self.config.schema == "seacrowd_sptext":
            features = schemas.speech_text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        lang, schema = self.config.subset_id.split("_", maxsplit=1)
        data_dir = dl_manager.download_and_extract(_DATA_URL)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # Whatever you put in gen_kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "data", lang, "text.txt"),
                    "audiopath": Path(os.path.join(data_dir, "data", lang, "audio")),
                },
            )
        ]

    def _generate_examples(self, filepath: Path, audiopath: Path) -> Tuple[int, Dict]:

        audiofiles = {}
        for audiofile in audiopath.iterdir():
            audio_idx = os.path.basename(audiofile).split(".")[0]
            audiofiles[audio_idx] = audiofile

        if self.config.schema == "source":
            for line_idx, line in enumerate(open(filepath)):
                audio_idx, text = line.strip().split(maxsplit=1)
                yield line_idx, {"id": line_idx, "text": text, "audio": str(audiofiles[audio_idx])}

        elif self.config.schema == "seacrowd_sptext":
            for line_idx, line in enumerate(open(filepath)):
                audio_idx, text = line.strip().split(maxsplit=1)
                yield line_idx, {"id": line_idx, "path": str(audiofiles[audio_idx]), "audio": str(audiofiles[audio_idx]), "text": text, "speaker_id": None, "metadata": {"speaker_age": None, "speaker_gender": None}}



"""
SEA Crowd Data Loader for SEA MADLAD.
"""

import gzip
import json
from typing import Dict, List, Tuple

import datasets
from datasets.download.download_manager import DownloadManager

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import TASK_TO_SCHEMA, Licenses, Tasks

_CITATION = r"""
@misc{kudugunta2023madlad400,
      title={MADLAD-400: A Multilingual And Document-Level Large Audited Dataset},
      author={Sneha Kudugunta and Isaac Caswell and Biao Zhang and Xavier Garcia and Christopher A. Choquette-Choo and Katherine Lee and Derrick Xin and Aditya Kusupati and Romi Stella and Ankur Bapna and Orhan Firat},
      year={2023},
      eprint={2309.04662},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}
"""

logger = datasets.logging.get_logger(__name__)

# this config is created for SEACrowd Dataloader
_LANG_CONFIG = {
    "ace": {"name": "Aceh", "source_subset": "ace"},
    "akb": {"name": "Batak Angkola", "source_subset": "akb"},
    "ban": {"name": "Bali", "source_subset": "ban"},
    "bbc": {"name": "Batak Toba", "source_subset": "bbc"},
    "bew": {"name": "Betawi", "source_subset": "bew"},
    "btx": {"name": "Batak Karo", "source_subset": "btx"},
    "ceb": {"name": "Cebuano", "source_subset": "ceb"},
    "fil": {"name": "Filipino", "source_subset": "fil"},
    "gor": {"name": "Gorontalo", "source_subset": "gor"},
    "hil": {"name": "Hiligaynon", "source_subset": "hil"},
    "iba": {"name": "Iban", "source_subset": "iba"},
    "ilo": {"name": "Ilocano", "source_subset": "ilo"},
    "ind": {"name": "Indonesian", "source_subset": "id"},
    "jav": {"name": "Javanese", "source_subset": "jv"},
    "kac": {"name": "Jingpho", "source_subset": "kac"},
    "khm": {"name": "Khmer", "source_subset": "km"},
    "kxd": {"name": "Brunei", "source_subset": "ms_Arab_BN"},
    "lao": {"name": "Lao", "source_subset": "lo"},
    "mad": {"name": "Madura", "source_subset": "mad"},
    "mak": {"name": "Makasar", "source_subset": "mak"},
    "meo": {"name": "Kedah Malay", "source_subset": "meo"},
    "min": {"name": "Minangkabau", "source_subset": "min"},
    "mkn": {"name": "Kupang Malay", "source_subset": "mkn"},
    "msa": {"name": "Malay", "source_subset": "ms"},
    "msi": {"name": "Sabah Malay", "source_subset": "msi"},
    "mya": {"name": "Burmese", "source_subset": "my"},
    "nij": {"name": "Ngaju", "source_subset": "nij"},
    "nut": {"name": "Nung", "source_subset": "nut"},
    "pag": {"name": "Pangasinan", "source_subset": "pag"},
    "shn": {"name": "Shan", "source_subset": "shn"},
    "sun": {"name": "Sunda", "source_subset": "su"},
    "tet": {"name": "Tetun", "source_subset": "tet"},
    "tha": {"name": "Thai", "source_subset": "th"},
    "vie": {"name": "Vietnamese", "source_subset": "vi"},
    "war": {"name": "Waray-Waray", "source_subset": "war"},
}

# this config is copied and added from source dataloader
# only using the `clean` values
_N_SHARDS_PER_SPLIT = {
    "ace": 1,
    "akb": 1,
    "ban": 1,
    "bbc": 1,
    "bew": 1,
    "btx": 1,
    "ceb": 1,
    "fil": 1,
    "gor": 1,
    "hil": 1,
    "iba": 1,
    "id": 18,
    "ilo": 1,
    "jv": 1,
    "kac": 1,
    "km": 1,
    "lo": 1,
    "mad": 1,
    "mak": 1,
    "meo": 1,
    "min": 1,
    "mkn": 1,
    "ms": 2,
    "ms_Arab_BN": 1,
    "msi": 1,
    "my": 1,
    "nij": 1,
    "nut": 1,
    "pag": 1,
    "shn": 1,
    "su": 1,
    "tet": 1,
    "th": 21,
    "vi": 32,
    "war": 1,
}

_LOCAL = False
_LANGUAGES = list(_LANG_CONFIG.keys())


_DATASETNAME = "sea_madlad"
_DESCRIPTION = r"""
    SEA MADLAD is a subset of MADLAD-400 (Multilingual Audited Dataset: Low-resource And Document-level), which is a document-level multilingual dataset based on Common Crawl.
    SEA MADLAD only filters the language of the "clean" subset, which covers 36 languages indigenous to SEA from 419 languages in total.
    As a result, some of SEA lang codes aren't available in this version because those belongs to the languages whose decision was to "remove from its clean version" based on MADLAD auditing process.
    MADLAD uses all snapshots of CommonCrawl available as of August 1, 2022.
    The primary advantage of this dataset over similar datasets is that it is more multilingual, it is audited and more highly filtered, and it is document-level.
    The main disadvantage is also its strength -- being more filtered, it may lack the recall needed for some applications.
"""

_HOMEPAGE = "https://huggingface.co/datasets/allenai/MADLAD-400"
_LICENSE = Licenses.CC_BY_4_0.value

_URL = "https://huggingface.co/datasets/allenai/MADLAD-400/resolve/ecd71297d60c1eb996cd3d7c44c60ad5b55adfc6/data/{language}/{language}_{split}_{index:04d}.jsonl.gz"

_SUPPORTED_TASKS = [Tasks.SELF_SUPERVISED_PRETRAINING]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"

CONFIG_SUFFIXES_FOR_TASK = [TASK_TO_SCHEMA.get(task).lower() for task in _SUPPORTED_TASKS]


def conform_init_config():
    """Assertion Function for Instantiated Configs"""
    if len(_LANGUAGES) == 0:
        raise AssertionError("No Languages detected from config!")
    if len(CONFIG_SUFFIXES_FOR_TASK) != len(_SUPPORTED_TASKS):
        raise AssertionError("Config prefixes don't matched in terms of `len` with `_SUPPORTED_TASKS`!")
    if len(CONFIG_SUFFIXES_FOR_TASK) == 0:
        raise AssertionError("Config prefixes and `_SUPPORTED_TASKS` have `len` of 0!")


conform_init_config()


def construct_configs_on_langs(languages: list = None) -> List[SEACrowdConfig]:
    """
    The function `construct_configs` constructs a list of SEACrowdConfig objects based on the provided
    languages or a default language, and returns the list.

    input:
        languages (list, default None): The `languages` parameter is a list that specifies the languages for which the
        configurations need to be constructed. If no languages are provided (value=None), the first value in language config
        will be used.
    output:
        a list of `SEACrowdConfig` objects based on instantiated init variables
    """

    # set output var
    config_list = []

    # construct zipped arg for config instantiation
    TASKS_AND_CONFIG_SUFFIX_PAIRS = list(zip(_SUPPORTED_TASKS, CONFIG_SUFFIXES_FOR_TASK))

    # implement source schema
    version, config_name_prefix = _SOURCE_VERSION, "source"
    config_list += [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{_LANG}_{config_name_prefix}",
            version=datasets.Version(version),
            description=f"{_DATASETNAME} {config_name_prefix} schema for language code {_LANG}",
            schema=f"{config_name_prefix}",
            subset_id=_LANG,
        )
        for _LANG in languages
    ]

    # implement SEACrowd schema
    version, config_name_prefix = _SEACROWD_VERSION, "seacrowd"
    for task_obj, config_name_suffix in TASKS_AND_CONFIG_SUFFIX_PAIRS:
        config_list += [
            SEACrowdConfig(
                name=f"{_DATASETNAME}_{_LANG}_{config_name_prefix}_{config_name_suffix}",
                version=datasets.Version(version),
                description=f"{_DATASETNAME} {config_name_prefix} schema for {task_obj.name} and language code {_LANG}",
                schema=f"{config_name_prefix}_{config_name_suffix}",
                subset_id=_LANG,
            )
            for _LANG in languages
        ]
    return config_list


class SEAMADLADDataset(datasets.GeneratorBasedBuilder):
    """SEA MADLAD dataset, subsetted from https://huggingface.co/datasets/allenai/MADLAD-400"""

    # get all schema w/o lang arg + get all schema w/ lang arg
    BUILDER_CONFIGS = construct_configs_on_langs(_LANGUAGES)

    def _info(self) -> datasets.DatasetInfo:
        _config_schema_name = self.config.schema
        logger.info(f"Received schema name: {self.config.schema}")
        # self supervised training schema
        if _config_schema_name == "source":
            features = datasets.Features({"text": datasets.Value("string")})

        elif _config_schema_name == "seacrowd_ssp":
            features = schemas.ssp_features

        else:
            raise ValueError(f"Received unexpected config schema of {_config_schema_name}!")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: DownloadManager) -> List[datasets.SplitGenerator]:
        # construct URL from "lang", "split" -> "clean" split, and "index" based on `_N_SHARDS_PER_SPLIT`
        _lang = _LANG_CONFIG[self.config.subset_id]["source_subset"]
        _split = "clean"
        _data_list = [_URL.format(language=_lang, split=_split, index=idx) for idx in range(_N_SHARDS_PER_SPLIT[_lang])]

        filepaths = dl_manager.download(_data_list)

        return [datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepaths": filepaths})]

    def _generate_examples(self, filepaths) -> Tuple[int, Dict]:
        _config_schema_name = self.config.schema

        # the id_ constructions follows the source Dataloader
        id_ = 0
        for filepath in filepaths:
            logger.info("generating examples from = %s", filepath)
            with gzip.open(open(filepath, "rb"), "rt", encoding="utf-8") as f:
                for line in f:
                    if line:
                        example = json.loads(line)

                        # for source_schema
                        if _config_schema_name == "source":
                            yield id_, {colname: example[colname] for colname in self.info.features}

                        # for ssp schema
                        elif _config_schema_name == "seacrowd_ssp":
                            yield id_, {"id": id_, "text": example["text"]}

                        else:
                            raise ValueError(f"Received unexpected config schema of {_config_schema_name}!")

                        id_ += 1



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This template serves as a starting point for contributing a dataset to the Nusantara Dataset repo.

When modifying it for your dataset, look for TODO items that offer specific instructions.

Full documentation on writing dataset loading scripts can be found here:
https://huggingface.co/docs/datasets/add_dataset.html

To create a dataset loading script you will create a class and implement 3 methods:
  * `_info`: Establishes the schema for the dataset, and returns a datasets.DatasetInfo object.
  * `_split_generators`: Downloads and extracts data for each split (e.g. train/val/test) or associate local data with each split.
  * `_generate_examples`: Creates examples from data on disk that conform to each schema defined in `_info`.

TODO: Before submitting your script, delete this doc string and replace it with a description of your dataset.

[seacrowd_schema_name] = (kb, pairs, qa, text, t2t, entailment)
"""
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.common_parser import load_ud_data, load_ud_data_as_seacrowd_kb
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{mcdonald-etal-2013-universal,
    title = "{U}niversal {D}ependency Annotation for Multilingual Parsing",
    author = {McDonald, Ryan  and
      Nivre, Joakim  and
      Quirmbach-Brundage, Yvonne  and
      Goldberg, Yoav  and
      Das, Dipanjan  and
      Ganchev, Kuzman  and
      Hall, Keith  and
      Petrov, Slav  and
      Zhang, Hao  and
      T{\"a}ckstr{\"o}m, Oscar  and
      Bedini, Claudia  and
      Bertomeu Castell{\'o}, N{\'u}ria  and
      Lee, Jungmee},
    booktitle = "Proceedings of the 51st Annual Meeting of the Association for Computational Linguistics (Volume 2: Short Papers)",
    month = aug,
    year = "2013",
    address = "Sofia, Bulgaria",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/P13-2017",
    pages = "92--97",
}

@article{DBLP:journals/corr/abs-2011-00677,
    author    = {Fajri Koto and
                 Afshin Rahimi and
                 Jey Han Lau and
                 Timothy Baldwin},
    title     = {IndoLEM and IndoBERT: {A} Benchmark Dataset and Pre-trained Language
                 Model for Indonesian {NLP}},
    journal   = {CoRR},
    volume    = {abs/2011.00677},
    year      = {2020},
    url       = {https://arxiv.org/abs/2011.00677},
    eprinttype = {arXiv},
    eprint    = {2011.00677},
    timestamp = {Fri, 06 Nov 2020 15:32:47 +0100},
    biburl    = {https://dblp.org/rec/journals/corr/abs-2011-00677.bib},
    bibsource = {dblp computer science bibliography, https://dblp.org}
}
"""


_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_DATASETNAME = "indolem_ud_id_gsd"

_DESCRIPTION = """\
The Indonesian-GSD treebank consists of 5598 sentences and 122k words split into train/dev/test of 97k/12k/11k words.
The treebank was originally converted from the content head version of the universal dependency treebank v2.0 (legacy) in 2015.\
In order to comply with the latest Indonesian annotation guidelines, the treebank has undergone a major revision between UD releases v2.8 and v2.9 (2021).
"""

_HOMEPAGE = "https://indolem.github.io/"

_LICENSE = "Creative Commons Attribution 4.0"

_URLS = {
    _DATASETNAME: {
        "train": "https://raw.githubusercontent.com/indolem/indolem/main/dependency_parsing/UD_Indonesian_GSD/id_gsd-ud-train.conllu",
        "validation": "https://raw.githubusercontent.com/indolem/indolem/main/dependency_parsing/UD_Indonesian_GSD/id_gsd-ud-dev.conllu",
        "test": "https://raw.githubusercontent.com/indolem/indolem/main/dependency_parsing/UD_Indonesian_GSD/id_gsd-ud-test.conllu",
    },
}

_SUPPORTED_TASKS = [Tasks.DEPENDENCY_PARSING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


# TODO: Name the dataset class to match the script name using CamelCase instead of snake_case
class IndolemUdIdGsdDataset(datasets.GeneratorBasedBuilder):
    """The Indonesian-GSD treebank, part of Universal-Dependency project. Consists of 5598 sentences and 122k words."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_kb",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} Nusantara KB schema",
            schema="seacrowd_kb",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    # metadata
                    "sent_id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    # tokens
                    "id": [datasets.Value("string")],
                    "form": [datasets.Value("string")],
                    "lemma": [datasets.Value("string")],
                    "upos": [datasets.Value("string")],
                    "xpos": [datasets.Value("string")],
                    "feats": [datasets.Value("string")],
                    "head": [datasets.Value("string")],
                    "deprel": [datasets.Value("string")],
                    "deps": [datasets.Value("string")],
                    "misc": [datasets.Value("string")],
                }
            )

        elif self.config.schema == "seacrowd_kb":
            features = schemas.kb_features

        else:
            raise NotImplementedError(f"Schema '{self.config.schema}' is not defined.")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[self.config.subset_id]
        data_dir = dl_manager.download(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir["train"],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_dir["test"],
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": data_dir["validation"],
                },
            ),
        ]

    def _generate_examples(self, filepath: Path) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        # method parameters are unpacked from `gen_kwargs` as given in `_split_generators`

        try:
            generator_fn = {
                "source": load_ud_data,
                "seacrowd_kb": load_ud_data_as_seacrowd_kb,
            }[self.config.schema]
        except KeyError:
            raise NotImplementedError(f"Schema '{self.config.schema}' is not defined.")

        for key, example in enumerate(generator_fn(filepath)):
            yield key, example


# if __name__ == "__main__":
#     datasets.load_dataset(__file__)



from pathlib import Path
from typing import List

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_DATASETNAME = "sap_wat"

_LANGUAGES = ["eng", "ind", "zlm", "tha", "vie"] 

_CITATION = """\
@inproceedings{buschbeck-exel-2020-parallel,
    title = "A Parallel Evaluation Data Set of Software Documentation with Document Structure Annotation",
    author = "Buschbeck, Bianka  and
      Exel, Miriam",
    editor = "Nakazawa, Toshiaki  and
      Nakayama, Hideki  and
      Ding, Chenchen  and
      Dabre, Raj  and
      Kunchukuttan, Anoop  and
      Pa, Win Pa  and
      Bojar, Ond{\v{r}}ej  and
      Parida, Shantipriya  and
      Goto, Isao  and
      Mino, Hidaya  and
      Manabe, Hiroshi  and
      Sudoh, Katsuhito  and
      Kurohashi, Sadao  and
      Bhattacharyya, Pushpak",
    booktitle = "Proceedings of the 7th Workshop on Asian Translation",
    month = dec,
    year = "2020",
    address = "Suzhou, China",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2020.wat-1.20",
    pages = "160--169",
    abstract = "This paper accompanies the software documentation data set for machine translation, a parallel 
    evaluation data set of data originating from the SAP Help Portal, that we released to the machine translation 
    community for research purposes. It offers the possibility to tune and evaluate machine translation systems 
    in the domain of corporate software documentation and contributes to the availability of a wider range of 
    evaluation scenarios. The data set comprises of the language pairs English to Hindi, Indonesian, Malay and 
    Thai, and thus also increases the test coverage for the many low-resource language pairs. Unlike most evaluation 
    data sets that consist of plain parallel text, the segments in this data set come with additional metadata that 
    describes structural information of the document context. We provide insights into the origin and creation, the 
    particularities and characteristics of the data set as well as machine translation results.",
}

"""

_DESCRIPTION = """The data set originates from the SAP Help Portal that contains documentation for SAP products and user 
assistance for product-related questions. The data has been processed in a way that makes it suitable as development and 
test data for machine translation purposes. The current language scope is English to Hindi, Indonesian, Japanese, Korean, 
Malay, Thai, Vietnamese, Simplified Chinese and Traditional Chinese. For each language pair about 4k segments are available, 
split into development and test data. The segments are provided in their document context and are annotated with additional 
metadata from the document."""

_HOMEPAGE = "https://github.com/SAP/software-documentation-data-set-for-machine-translation"

_LICENSE = Licenses.CC_BY_NC_4_0.value

_URLs = {
    _DATASETNAME: "https://raw.githubusercontent.com/SAP/software-documentation-data-set-for-machine-translation/master/{split}_data/en{lang}/software_documentation.{split}.en{lang}.{appx}"
}

_SUPPORTED_TASKS = [
    Tasks.MACHINE_TRANSLATION
]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"

_SUBSET = ["id", "ms", "th", "vi"]

_LOCAL = False

class SapWatDataset(datasets.GeneratorBasedBuilder):
    """SAP WAT is a software documentation dataset for machine translation. The current language scope is English to Hindi, 
    Indonesian, Japanese, Korean, Malay, Thai, Vietnamese, Simplified Chinese and Traditional Chinese. Here, we only consider 
    EN-ID, EN-TH, EN-MS, EN-VI"""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_en_{lang}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"SAP WAT source schema for EN-{lang.upper()}",
            schema="source",
            subset_id=f"{_DATASETNAME}_en_{lang}",
        ) 
        for lang in _SUBSET] + [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_en_{lang}_seacrowd_t2t",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"SAP WAT SEACrowd schema for EN-{lang.upper()}",
            schema="seacrowd_t2t",
            subset_id=f"{_DATASETNAME}_en_{lang}",
        ) 
        for lang in _SUBSET
    ]

    DEFAULT_CONFIG_NAME = "sap_wat_en_id_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                    "label": datasets.Value("string")
                }
            )
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(
        self, dl_manager: datasets.DownloadManager
    ) -> List[datasets.SplitGenerator]:
        lang = self.config.name.split("_")[3]

        splits = {datasets.Split.VALIDATION: "dev", datasets.Split.TEST: "test"}
        data_urls = {
            split: _URLs[_DATASETNAME].format(split=splits[split], lang=lang, appx=lang) for split in splits
        }
        dl_paths = dl_manager.download(data_urls)

        en_data_urls = {
            split: _URLs[_DATASETNAME].format(split=splits[split], lang=lang, appx="en") for split in splits
        }
        en_dl_paths = dl_manager.download(en_data_urls)
        return [
            datasets.SplitGenerator(
                name=split,
                gen_kwargs={"filepath": dl_paths[split], "en_filepath": en_dl_paths[split]},
            )
            for split in splits
        ]

    def _generate_examples(self, filepath: Path, en_filepath: Path):
        with open(en_filepath, "r") as f:
            lines_1 = f.readlines()
        with open(filepath, "r") as f:
            lines_2 = f.readlines()

        if self.config.schema == "source":
            for _id, (line_1, line_2) in enumerate(zip(lines_1, lines_2)):
                ex = {
                    "id": _id,
                    "text": line_1.strip(),
                    "label": line_2.strip()
                }                
                yield _id, ex

        elif self.config.schema == "seacrowd_t2t":
            lang = self.config.name.split("_")[3]
            lang_name = _LANGUAGES[_SUBSET.index(lang)+1]

            for _id, (line_1, line_2) in enumerate(zip(lines_1, lines_2)):
                ex = {
                    "id": _id,
                    "text_1": line_1.strip(),
                    "text_2": line_2.strip(),
                    "text_1_name": 'eng',
                    "text_2_name": lang_name,
                }
                yield _id, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")


# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from itertools import chain
from pathlib import Path
from random import sample
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{sakti-cocosda-2013,
    title = "Towards Language Preservation: Design and Collection of Graphemically Balanced and Parallel Speech Corpora of {I}ndonesian Ethnic Languages",
    author = "Sakti, Sakriani and Nakamura, Satoshi",
    booktitle = "Proc. Oriental COCOSDA",
    year = "2013",
    address = "Gurgaon, India"
}

@inproceedings{sakti-sltu-2014,
  title = "Recent progress in developing grapheme-based speech recognition for {I}ndonesian ethnic languages: {J}avanese, {S}undanese, {B}alinese and {B}ataks",
  author = "Sakti, Sakriani and Nakamura, Satoshi",
  booktitle = "Proc. 4th Workshop on Spoken Language Technologies for Under-Resourced Languages (SLTU 2014)",
  year = "2014",
  pages = "46--52",
  address = "St. Petersburg, Russia"
}

@inproceedings{novitasari-sltu-2020,
  title = "Cross-Lingual Machine Speech Chain for {J}avanese, {S}undanese, {B}alinese, and {B}ataks Speech Recognition and Synthesis",
  author = "Novitasari, Sashi and Tjandra, Andros and Sakti, Sakriani and Nakamura, Satoshi",
  booktitle = "Proc. Joint Workshop on Spoken Language Technologies for Under-resourced languages (SLTU) and Collaboration and Computing for Under-Resourced Languages (CCURL)",
  year = "2020",
  pages = "131--138",
  address = "Marseille, France"
}
"""

_DATASETNAME = "indspeech_newstra_ethnicsr"
_DESCRIPTION = """\
INDspeech_NEWSTRA_EthnicSR is a collection of graphemically balanced and parallel speech corpora of four major Indonesian ethnic languages: Javanese, Sundanese, Balinese, and Bataks. It was developed in 2013 by the Nara Institute of Science and Technology (NAIST, Japan) [Sakti et al., 2013]. The data has been used to develop Indonesian ethnic speech recognition in supervised learning [Sakti et al., 2014] and semi-supervised learning [Novitasari et al., 2020] based on Machine Speech Chain framework [Tjandra et al., 2020].
"""

_HOMEPAGE = "https://github.com/s-sakti/data_indsp_newstra_ethnicsr"
_LANGUAGES = ["sun", "jav", "btk", "ban"]
_LOCAL = False
_LICENSE = "CC-BY-NC-SA 4.0"

_lst_TYPE = ["traEth", "traInd"]
_lst_LANG = {"Bli": "BALI", "Btk": "BATAK", "Jaw": "JAWA", "Snd": "SUNDA"}
_lst_STD_LANG = {"ban": "Bli", "btk": "Btk", "jav": "Jaw", "sun": "Snd"}
_lst_HEAD_1_TRAIN = "https://raw.githubusercontent.com/s-sakti/data_indsp_newstra_ethnicsr/main/lst/dataset1_train_news_"
_lst_HEAD_1_TEST = ["https://raw.githubusercontent.com/s-sakti/data_indsp_newstra_ethnicsr/main/lst/dataset1_test_" + ltype + "_" for ltype in _lst_TYPE]
_lst_HEAD_2 = "https://raw.githubusercontent.com/s-sakti/data_indsp_newstra_ethnicsr/main/lst/dataset2_"
_sp_TEMPLATE = "https://raw.githubusercontent.com/s-sakti/data_indsp_newstra_ethnicsr/main/speech/16kHz/"  # {lang}/Ind0{index}_{gender}_{lang_code}.zip"
_txt_TEMPLATE = "https://github.com/s-sakti/data_indsp_newstra_ethnicsr/raw/main/text/utts_transcript/"  # {lang}/Ind0{index}_{gender}_{lang_code}.zip"

_URLS = {
    "dataset1_train": {llang.lower(): [_lst_HEAD_1_TRAIN + llang + ".lst"] for llang in _lst_LANG},
    "dataset1_test": {llang.lower(): [head1test + llang + ".lst" for head1test in _lst_HEAD_1_TEST] for llang in _lst_LANG},
    "dataset2_train": {llang.lower(): [_lst_HEAD_2 + "train_news_" + llang + ".lst"] for llang in _lst_LANG},
    "dataset2_test": {llang.lower(): [_lst_HEAD_2 + "test_news_" + llang + ".lst"] for llang in _lst_LANG},
    "speech": {llang.lower(): [_sp_TEMPLATE + _lst_LANG[llang] + "/Ind" + str(idx).zfill(3) + "_" + ("M" if idx % 2 == 0 else "F") + "_" + llang + ".zip" for idx in range(1, 11)] for llang in _lst_LANG},
    "transcript": {llang.lower(): [_txt_TEMPLATE + _lst_LANG[llang] + "/Ind" + str(idx).zfill(3) + "_" + ("M" if idx % 2 == 0 else "F") + "_" + llang + ".zip" for idx in range(1, 11)] for llang in _lst_LANG},
}

_SUPPORTED_TASKS = [Tasks.SPEECH_RECOGNITION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


def seacrowd_config_constructor(lang, schema, version, overlap):
    if lang == "":
        raise ValueError(f"Invalid lang {lang}")

    if schema != "source" and schema != "seacrowd_sptext":
        raise ValueError(f"Invalid schema: {schema}")

    return SEACrowdConfig(
        name="indspeech_newstra_ethnicsr_{overlap}_{lang}_{schema}".format(lang=lang, schema=schema, overlap=overlap),
        version=datasets.Version(version),
        description="indspeech_newstra_ethnicsr {schema} schema for {lang} language with {overlap}ping dataset".format(lang=_lst_LANG[_lst_STD_LANG[lang]], schema=schema, overlap=overlap),
        schema=schema,
        subset_id="indspeech_newstra_ethnicsr_{overlap}".format(overlap=overlap),
    )

class INDspeechNEWSTRAEthnicSR(datasets.GeneratorBasedBuilder):
    """
    The dataset contains 2 sub-datasets
    Dataset 1 has 2250/1000 train/test samples per language
    Dataset 2 has another 1600/50 train/test per language
    The 'overlap' keyword in the dataset-name combines both sub-datasets, while 'nooverlap' will only use dataset 1
    """
    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [seacrowd_config_constructor(lang, "source", _SOURCE_VERSION, overlap) for lang in _lst_STD_LANG for overlap in ["overlap","nooverlap"]] +\
                      [seacrowd_config_constructor(lang, "seacrowd_sptext", _SEACROWD_VERSION, overlap) for lang in _lst_STD_LANG for overlap in ["overlap","nooverlap"]]

    DEFAULT_CONFIG_NAME = "indspeech_newstra_ethnicsr_jav_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "speaker_id": datasets.Value("string"),
                    "path": datasets.Value("string"),
                    "audio": datasets.Audio(sampling_rate=16_000),
                    "text": datasets.Value("string"),
                    "gender": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_sptext":
            features = schemas.speech_text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        lang = _lst_STD_LANG[self.config.name.split("_")[4]].lower()
        ds1_train_urls = _URLS["dataset1_train"][lang]
        ds1_test_urls = _URLS["dataset1_test"][lang]
        ds2_train_urls = _URLS["dataset2_train"][lang]
        ds2_test_urls = _URLS["dataset2_test"][lang]
        sp_urls = _URLS["speech"][lang]
        txt_urls = _URLS["transcript"][lang]

        ds1_train_dir = [Path(dl_manager.download_and_extract(ds1_train_url)) for ds1_train_url in ds1_train_urls]
        ds1_test_dir = [Path(dl_manager.download_and_extract(ds1_test_url)) for ds1_test_url in ds1_test_urls]
        ds2_train_dir = [Path(dl_manager.download_and_extract(ds2_train_url)) for ds2_train_url in ds2_train_urls]
        ds2_test_dir = [Path(dl_manager.download_and_extract(ds2_test_url)) for ds2_test_url in ds2_test_urls]
        sp_dir = {str(Path(sp_url).name)[:-4]: os.path.join(Path(dl_manager.download_and_extract(sp_url)), str(Path(sp_url).name))[:-4] for sp_url in sp_urls}
        txt_dir = {str(Path(txt_url).name)[:-4]: os.path.join(Path(dl_manager.download_and_extract(txt_url)), str(Path(txt_url).name))[:-4] for txt_url in txt_urls}

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": {
                        "dataset1": ds1_train_dir,
                        "dataset2": ds2_train_dir,
                        "speech": sp_dir,
                        "transcript": txt_dir,
                    },
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": {
                        "dataset1": ds1_test_dir,
                        "dataset2": ds2_test_dir,
                        "speech": sp_dir,
                        "transcript": txt_dir,
                    },
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        sample_list=[]
        if self.config.name.split("_")[3] == "nooverlap":
            sample_list = [open(samples).read().splitlines() for samples in filepath["dataset1"]]  # +filepath["dataset2"]]
            sample_list = list(chain(*sample_list))
        elif self.config.name.split("_")[3] == "overlap":
            sample_list = [open(samples).read().splitlines() for samples in filepath["dataset1"]+filepath["dataset2"]]
            sample_list = list(chain(*sample_list))

        for id, row in enumerate(sample_list):
            if self.config.schema == "source":
                ex = {
                    "id": str(id),
                    "speaker_id": str(Path(row).parent).split("/")[1],
                    "path": os.path.join(filepath["speech"][str(Path(row).parent).split("/")[1]], str(Path(row).name) + ".wav"),
                    "audio": os.path.join(filepath["speech"][str(Path(row).parent).split("/")[1]], str(Path(row).name) + ".wav"),
                    "text": open(os.path.join(filepath["transcript"][str(Path(row).parent).split("/")[1]], str(Path(row).name) + ".txt"), "r").read().splitlines()[0],
                    "gender": str(Path(row).parent).split("/")[1].split("_")[1],
                }
                yield id, ex

            elif self.config.schema == "seacrowd_sptext":
                ex = {
                    "id": str(id),
                    "speaker_id": str(Path(row).parent).split("/")[1],
                    "path": os.path.join(filepath["speech"][str(Path(row).parent).split("/")[1]], str(Path(row).name) + ".wav"),
                    "audio": os.path.join(filepath["speech"][str(Path(row).parent).split("/")[1]], str(Path(row).name) + ".wav"),
                    "text": open(os.path.join(filepath["transcript"][str(Path(row).parent).split("/")[1]], str(Path(row).name) + ".txt"), "r").read().splitlines()[0],
                    "metadata": {
                        "speaker_age": None,
                        "speaker_gender": str(Path(row).parent).split("/")[1].split("_")[1],
                    },
                }
                yield id, ex



# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import csv
import json
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@misc{lent2023creoleval,
    title={CreoleVal: Multilingual Multitask Benchmarks for Creoles},
    author={Heather Lent and Kushal Tatariya and Raj Dabre and Yiyi Chen and Marcell Fekete and Esther Ploeger and Li Zhou and Hans Erik Heje and Diptesh Kanojia and Paul Belony and Marcel Bollmann and \
    Loïc Grobol and Miryam de Lhoneux and Daniel Hershcovich and Michel DeGraff and Anders Søgaard and Johannes Bjerva},
    year={2023},
    eprint={2310.19567},
    archivePrefix={arXiv},
    primaryClass={cs.CL}
}
"""

_DATASETNAME = "creole_rc"
_DESCRIPTION = """\
CreoleRC is a subset created by the CreoleVal paper. Relation classification (RC) aims to identify semantic associations between entities within a text, essential for applications like knowledge base \
completion and question answering. The dataset is sourced from Wikipedia and manually annotated. CreoleRC contains 5 creoles, but SEACrowd is interested specifically in the Chavacano subset.
"""
_HOMEPAGE = "https://github.com/hclent/CreoleVal/tree/main/nlu/relation_classification"
_LANGUAGES = ["cbk"]
_LICENSE = Licenses.CC_BY_SA_4_0.value
_LOCAL = False
_URLS = {
    "csv": "https://raw.githubusercontent.com/hclent/CreoleVal/main/nlu/relation_classification/data/relation_extraction/cbk-zam.csv",
    "json": "https://raw.githubusercontent.com/hclent/CreoleVal/main/nlu/relation_classification/data/relation_extraction/cbk-zam.json",
}
_SUPPORTED_TASKS = [Tasks.RELATION_EXTRACTION]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class CreoleRC(datasets.GeneratorBasedBuilder):
    """Creole relation classification dataset, Chavacano subset, from https://github.com/hclent/CreoleVal/tree/main/nlu/relation_classification."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    # You will be able to load the "source" or "seacrowd" configurations with
    # ds_source = datasets.load_dataset('my_dataset', name='source')
    # ds_seacrowd = datasets.load_dataset('my_dataset', name='seacrowd')

    # For local datasets you can make use of the `data_dir` and `data_files` kwargs
    # https://huggingface.co/docs/datasets/add_dataset.html#downloading-data-files-and-organizing-splits
    # ds_source = datasets.load_dataset('my_dataset', name='source', data_dir="/path/to/data/files")
    # ds_seacrowd = datasets.load_dataset('my_dataset', name='seacrowd', data_dir="/path/to/data/files")

    SEACROWD_SCHEMA_NAME = "kb"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=_DATASETNAME,
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=_DATASETNAME,
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "sentence": datasets.Value("string"),
                    "ent1": datasets.Value("string"),
                    "ent2": datasets.Value("string"),
                    "ent1_qcode": datasets.Value("string"),
                    "ent2_qcode": datasets.Value("string"),
                    "property": datasets.Value("string"),
                    "property_desc": datasets.Value("string"),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "edgeset_left": datasets.Sequence(datasets.Value("int32")),
                    "edgeset_right": datasets.Sequence(datasets.Value("int32")),
                    "edgeset_triple": datasets.Sequence(datasets.Value("string")),
                }
            )

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.kb_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        data_paths = {
            "csv": Path(dl_manager.download_and_extract(_URLS["csv"])),
            "json": Path(dl_manager.download_and_extract(_URLS["json"])),
        }

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "csv_filepath": data_paths["csv"],
                    "json_filepath": data_paths["json"],
                    "split": "train",
                },
            )
        ]

    # method parameters are unpacked from `gen_kwargs` as given in `_split_generators`

    def _generate_examples(self, csv_filepath: Path, json_filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        # read csv file
        with open(csv_filepath, "r", encoding="utf-8") as csv_file:
            csv_reader = csv.reader(csv_file)
            csv_data = [row for row in csv_reader]
        csv_data = csv_data[1:]  # remove header

        # read json file
        with open(json_filepath, "r", encoding="utf-8") as json_file:
            json_data = json.load(json_file)

        # properties descriptions from https://github.com/hclent/CreoleVal/tree/main/nlu/relation_classification
        # for properties present in Chavacano subset
        properties_desc = {"P17": "country", "P30": "continent", "P106": "occupation", "P131": "located in the administrative territorial entity", "P361": "part of", "P1376": " capital of country"}

        num_sample = len(csv_data)

        for i in range(num_sample):
            if self.config.schema == "source":
                example = {
                    "id": str(i),
                    "sentence": csv_data[i][0],
                    "ent1": csv_data[i][1],
                    "ent2": csv_data[i][2],
                    "ent1_qcode": csv_data[i][3],
                    "ent2_qcode": csv_data[i][4],
                    "property": csv_data[i][5],
                    "property_desc": properties_desc[csv_data[i][5]],
                    "tokens": json_data[i]["tokens"],
                    "edgeset_left": json_data[i]["edgeSet"]["left"],
                    "edgeset_right": json_data[i]["edgeSet"]["right"],
                    "edgeset_triple": json_data[i]["edgeSet"]["triple"],
                }
            elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
                offset_entity1 = csv_data[i][0].find(csv_data[i][1])
                offset_entity2 = csv_data[i][0].find(csv_data[i][2])

                if (offset_entity1 == -1) or (offset_entity2 == -1):
                    continue

                example = {
                    "id": str(i),
                    "passages": [{"id": f"passage-{i}", "type": "text", "text": [csv_data[i][0]], "offsets": [[0, len(csv_data[i][0])]]}],
                    "entities": [
                        {"id": f"{i}-entity-{csv_data[i][3]}", "type": "text", "text": [csv_data[i][1]], "normalized": [{"db_name": csv_data[i][1], "db_id": csv_data[i][3]}], "offsets": [[offset_entity1, offset_entity1 + len(csv_data[i][1])]]},
                        {"id": f"{i}-entity-{csv_data[i][4]}", "type": "text", "text": [csv_data[i][2]], "normalized": [{"db_name": csv_data[i][2], "db_id": csv_data[i][4]}], "offsets": [[offset_entity2, offset_entity2 + len(csv_data[i][2])]]},
                    ],
                    "events": [],
                    "coreferences": [],
                    "relations": [
                        {
                            "id": f"{i}-relation-{csv_data[i][5]}",
                            "type": properties_desc[csv_data[i][5]],
                            "arg1_id": f"{i}-entity-{csv_data[i][3]}",
                            "arg2_id": f"{i}-entity-{csv_data[i][4]}",
                            "normalized": [{"db_name": properties_desc[csv_data[i][5]], "db_id": csv_data[i][5]}],
                        }
                    ],
                }

            yield i, example



# coding=utf-8
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import TASK_TO_SCHEMA, Licenses, Tasks

_DATASETNAME = "melayu_sarawak"


_DESCRIPTION = """\
Korpus Variasi Bahasa Melayu: Sarawak is a language corpus sourced from various folklores in Melayu Sarawak dialect.
"""


_HOMEPAGE = "https://github.com/matbahasa/Melayu_Sarawak"

_CITATION = """\
@misc{melayusarawak,
  author = {Hiroki Nomoto},
  title = {Melayu_Sabah},
  year = {2020},
  publisher = {GitHub},
  journal = {GitHub repository},
  howpublished = {\\url{https://github.com/matbahasa/Melayu_Sarawak}},
  commit = {a175f691f9db94d7b4f971e7a93b7cc001c0ed47}
}
"""

_LANGUAGES = ["zlm"]

_LICENSE = Licenses.CC_BY_4_0.value

_LOCAL = False

_URLS = {
    "sarawak201801": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201801.txt",
    "sarawak201802": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201802.txt",
    "sarawak201803": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201803.txt",
    "sarawak201804": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201804.txt",
    "sarawak201805": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201805.txt",
    "sarawak201806": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201806.txt",
    "sarawak201807": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201807.txt",
    "sarawak201808": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201808.txt",
    "sarawak201809": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201809.txt",
    "sarawak201810": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201810.txt",
    "sarawak201811": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201811.txt",
    "sarawak201812": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201812.txt",
    "sarawak201813": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201813.txt",
    "sarawak201814": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201814.txt",
    "sarawak201815": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201815.txt",
    "sarawak201817": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201817.txt",
    "sarawak201818": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201818.txt",
    "sarawak201819": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201819.txt",
    "sarawak201820": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201820.txt",
    "sarawak201821": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201821.txt",
    "sarawak201822": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201822.txt",
    "sarawak201823": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201823.txt",
    "sarawak201824": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201824.txt",
    "sarawak201825": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201825.txt",
    "sarawak201826": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201826.txt",
    "sarawak201827": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201827.txt",
    "sarawak201828": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201828.txt",
    "sarawak201829": "https://raw.githubusercontent.com/matbahasa/Melayu_Sarawak/master/Sarawak201829.txt",
}

_SUPPORTED_TASKS = [Tasks.SELF_SUPERVISED_PRETRAINING]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class MelayuSarawakDataset(datasets.GeneratorBasedBuilder):
    """Korpus Variasi Bahasa Melayu:
    Sarawak is a language corpus sourced from various folklores in Melayu Sarawak dialect."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = TASK_TO_SCHEMA[_SUPPORTED_TASKS[0]].lower()

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "text": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_ssp":
            features = schemas.self_supervised_pretraining.features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""

        urls = [_URLS[key] for key in _URLS.keys()]
        data_path = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_path[0], "split": "train", "other_path": data_path[1:]},
            )
        ]

    def _generate_examples(self, filepath: Path, split: str, other_path: List) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        filepaths = [filepath] + other_path
        data = []
        for filepath in filepaths:
            with open(filepath, "r") as f:
                data.append([line.rstrip() for line in f.readlines()])

        for id, text in enumerate(data):
            yield id, {"id": id, "text": text}


from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, Licenses, TASK_TO_SCHEMA, SCHEMA_TO_FEATURES

_CITATION = """\
@inproceedings{kargaran2023glotlid,
    title     = {{GlotLID}: Language Identification for Low-Resource Languages},
    author    = {Kargaran, Amir Hossein and
                Imani, Ayyoob and
                Yvon, Fran{\c{c}}ois
                and Sch{\"u}tze, Hinrich},
    booktitle = {The 2023 Conference on Empirical Methods in Natural Language Processing},
    year      = {2023},
    url       = {https://openreview.net/forum?id=dl4e3EBz5j}
}
"""

_DATASETNAME = "glotstorybook"
_DESCRIPTION = """\
The GlotStoryBook dataset is a compilation of children's storybooks from the Global
Storybooks project, encompassing 174 languages organized for machine translation tasks. It
features rows containing the text segment (text number), the language code, and the file
name, which corresponds to the specific book and story segment. This structure allows for
the comparison of texts across different languages by matching file names and text numbers
between rows.
"""

_HOMEPAGE = "https://huggingface.co/datasets/cis-lmu/GlotStoryBook"
_LICENSE = f"""{Licenses.OTHERS.value} | \
We do not own any of the text from which these data has been extracted. All the files are
collected from the repository located at https://github.com/global-asp/. The source
repository for each text and file is stored in the dataset. Each file in the dataset is
associated with one license from the CC family. The licenses include 'CC BY', 'CC BY-NC',
'CC BY-NC-SA', 'CC-BY', 'CC-BY-NC', and 'Public Domain'. We also license the code, actual
packaging and the metadata of these data under the cc0-1.0.
"""

_LOCAL=False
_LANGUAGES = ["khg", "khm", "mya", "tet", "tha", "vie"]

_URLS = "https://huggingface.co/datasets/cis-lmu/GlotStoryBook/resolve/main/GlotStoryBook.csv"

_SUPPORTED_TASKS = [Tasks.SELF_SUPERVISED_PRETRAINING]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class GlotStoryBookDataset(datasets.GeneratorBasedBuilder):
    """Compilation of storybooks from the Global Storybooks project"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = TASK_TO_SCHEMA[_SUPPORTED_TASKS[0]].lower()

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "text_number": datasets.Value("int64"),
                    "license": datasets.Value("string"),
                    "text_by": datasets.Value("string"),
                    "translation_by": datasets.Value("string"),
                    "language": datasets.Value("string"),
                    "file_name": datasets.Value("string"),
                    "source": datasets.Value("string"),
                    "iso639-3": datasets.Value("string"),
                    "script": datasets.Value("string"),
                }
            )
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = SCHEMA_TO_FEATURES[self.SEACROWD_SCHEMA_NAME.upper()]

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        data_path = Path(dl_manager.download_and_extract(_URLS))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_path,
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        df = pd.read_csv(filepath)
        df = df[df["ISO639-3"].isin(_LANGUAGES)]

        if self.config.schema == "source":
            for i, row in df.iterrows():
                yield i, {
                    "text": row["Text"],
                    "text_number": row["Text Number"],
                    "license": row["License"],
                    "text_by": row["Text By"],
                    "translation_by": row["Translation By"],
                    "language": row["Language"],
                    "file_name": row["File Name"],
                    "source": row["Source"],
                    "iso639-3": row["ISO639-3"],
                    "script": row["Script"],
                }
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            df = df.sort_values(by=["ISO639-3", "Source", "File Name", "Text Number"])
            df = df.groupby(["ISO639-3", "Source", "File Name"]).agg({"Text": " ".join}).reset_index()
            for i, row in df.iterrows():
                yield i, {
                    "id": str(i),
                    "text": row["Text"],
                }


from pathlib import Path
from typing import Dict, List, Tuple

import datasets
import pandas as pd

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import (SCHEMA_TO_FEATURES, TASK_TO_SCHEMA,
                                      Licenses, Tasks)

_CITATION = """\
@article{hendria2023msvd,
    author    = {Willy Fitra Hendria},
    title     = {MSVD-Indonesian: A Benchmark for Multimodal Video-Text Tasks in Indonesian},
    journal   = {arXiv preprint arXiv:2306.11341},
    year      = {2023},
    url       = {https://arxiv.org/abs/2306.11341},
}
"""

_DATASETNAME = "id_msvd"
_DESCRIPTION = """\
MSVD-Indonesian is derived from the MSVD (Microsoft Video Description) dataset, which is
obtained with the help of a machine translation service (Google Translate API). This
dataset can be used for multimodal video-text tasks, including text-to-video retrieval,
video-to-text retrieval, and video captioning. Same as the original English dataset, the
MSVD-Indonesian dataset contains about 80k video-text pairs.
"""

_HOMEPAGE = "https://github.com/willyfh/msvd-indonesian"
_LANGUAGES = ["ind"]
_LICENSE = Licenses.MIT.value
_URLS = {"text": "https://raw.githubusercontent.com/willyfh/msvd-indonesian/main/data/MSVD-indonesian.txt", "video": "https://www.cs.utexas.edu/users/ml/clamp/videoDescription/YouTubeClips.tar"}

_SUPPORTED_TASKS = [Tasks.VIDEO_TO_TEXT_RETRIEVAL]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"

_LOCAL = False


class IdMsvdDataset(datasets.GeneratorBasedBuilder):
    """MSVD dataset with Indonesian translation."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = TASK_TO_SCHEMA[_SUPPORTED_TASKS[0]].lower()  # "vidtext"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "video_path": datasets.Value("string"),
                    "text": datasets.Value("string"),
                }
            )
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = SCHEMA_TO_FEATURES[self.SEACROWD_SCHEMA_NAME.upper()]  # video_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        # expect several minutes to download video data ~1.7GB
        data_path = dl_manager.download_and_extract(_URLS)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "text_path": Path(data_path["text"]),
                    "video_path": Path(data_path["video"]) / "YouTubeClips",
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, text_path: Path, video_path: Path, split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples."""
        text_data = []
        with open(text_path, "r", encoding="utf-8") as f:
            for line in f:
                id = line.find(" ")
                video = line[:id]
                text = line[id + 1 :].strip()
                text_data.append([video, text])

        df = pd.DataFrame(text_data, columns=["video_path", "text"])
        df["video_path"] = df["video_path"].apply(lambda x: video_path / f"{x}.avi")

        if self.config.schema == "source":
            for i, row in df.iterrows():
                yield i, {
                    "video_path": str(row["video_path"]),
                    "text": row["text"],
                }
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            for i, row in df.iterrows():
                yield i, {
                    "id": str(i),
                    "video_path": str(row["video_path"]),
                    "text": row["text"],
                    "metadata": {
                        "resolution": {
                            "width": None,
                            "height": None,
                        },
                        "duration": None,
                        "fps": None,
                    },
                }



"""
SpeechSpeech Schema
"""
import datasets

features = datasets.Features(
    {
        "id": datasets.Value("string"),
        "path_1": datasets.Value("string"),
        "audio_1": datasets.Audio(sampling_rate=16_000),
        "text_1": datasets.Value("string"),
        "metadata_1": {
            "name": datasets.Value("string"),
            "speaker_age": datasets.Value("int64"),
            "speaker_gender": datasets.Value("string"),
        },
        "path_2": datasets.Value("string"),
        "audio_2": datasets.Audio(sampling_rate=16_000),
        "text_2": datasets.Value("string"),
        "metadata_2": {
            "name": datasets.Value("string"),
            "speaker_age": datasets.Value("int64"),
            "speaker_gender": datasets.Value("string"),
        }
    }
)



from .utils.constants import Tasks
from .config_helper import list_datasets, load_dataset, load_datasets, list_benchmarks, load_benchmark
from .config_helper import SEACrowdMetadata, SEACrowdConfigHelper, SEACrowdMetadataHelper

__version__ = "0.0.5"



from pathlib import Path
from typing import Dict, List, Tuple

import datasets
from seacrowd.utils import schemas
from seacrowd.utils.common_parser import load_conll_data

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{koto-etal-2020-indolem,
    title = "{I}ndo{LEM} and {I}ndo{BERT}: A Benchmark Dataset and Pre-trained Language Model for {I}ndonesian {NLP}",
    author = "Koto, Fajri  and
      Rahimi, Afshin  and
      Lau, Jey Han  and
      Baldwin, Timothy",
    booktitle = "Proceedings of the 28th International Conference on Computational Linguistics",
    month = dec,
    year = "2020",
    address = "Barcelona, Spain (Online)",
    publisher = "International Committee on Computational Linguistics",
    url = "https://aclanthology.org/2020.coling-main.66",
    doi = "10.18653/v1/2020.coling-main.66",
    pages = "757--770"
}
@phdthesis{fachri2014pengenalan,
  title     = {Pengenalan Entitas Bernama Pada Teks Bahasa Indonesia Menggunakan Hidden Markov Model},
  author    = {FACHRI, MUHAMMAD},
  year      = {2014},
  school    = {Universitas Gadjah Mada}
}
"""

_LOCAL = False
_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_DATASETNAME = "indolem_ner_ugm"

_DESCRIPTION = """\
NER UGM is a Named Entity Recognition dataset that comprises 2,343 sentences from news articles, and was constructed at the University of Gajah Mada based on five named entity classes: person, organization, location, time, and quantity.
"""

_HOMEPAGE = "https://indolem.github.io/"

_LICENSE = "Creative Commons Attribution 4.0"

_URLS = {
    _DATASETNAME: {
        "train": "https://raw.githubusercontent.com/indolem/indolem/main/ner/data/nerugm/train.0{fold_number}.tsv",
        "validation": "https://raw.githubusercontent.com/indolem/indolem/main/ner/data/nerugm/dev.0{fold_number}.tsv",
        "test": "https://raw.githubusercontent.com/indolem/indolem/main/ner/data/nerugm/test.0{fold_number}.tsv"
    }
}

_SUPPORTED_TASKS = [Tasks.NAMED_ENTITY_RECOGNITION]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"

class IndolemNERUGM(datasets.GeneratorBasedBuilder):
    """NER UGM comprises 2,343 sentences from news articles, and was constructed at the University of Gajah Mada based on five named entity classes: person, organization, location, time, and quantity; and based on 5-fold cross validation"""

    label_classes = ["B-PERSON", "B-LOCATION", "B-ORGANIZATION", "B-TIME", "B-QUANTITY", "I-PERSON", "I-LOCATION", "I-ORGANIZATION", "I-TIME", "I-QUANTITY", "O"]

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = ( 
    [
        SEACrowdConfig(
            name="indolem_ner_ugm_fold{fold_number}_source".format(fold_number=i),
            version=_SOURCE_VERSION,
            description="indolem_ner_ugm source schema",
            schema="source",
            subset_id="indolem_ner_ugm_fold{fold_number}".format(fold_number=i),
        ) for i in range(5)
    ] 
    +   [
            SEACrowdConfig(
                name="indolem_ner_ugm_fold{fold_number}_seacrowd_seq_label".format(fold_number=i),
                version=_SEACROWD_VERSION,
                description="indolem_ner_ugm Nusantara schema",
                schema="seacrowd_seq_label",
                subset_id="indolem_ner_ugm_fold{fold_number}".format(fold_number=i),
            ) for i in range(5)
        ]
    )

    DEFAULT_CONFIG_NAME = "indolem_ner_ugm_fold0_source"

    def _info(self) -> datasets.DatasetInfo:

        if self.config.schema == "source":

            features = datasets.Features(
                {
                    "index": datasets.Value("string"),
                    "tokens": [datasets.Value("string")],
                    "tags": [datasets.Value("string")]
                }
            )

        elif self.config.schema == "seacrowd_seq_label":
            features = schemas.seq_label_features(self.label_classes)

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _get_fold_index(self):
        try:
            subset_id = self.config.subset_id
            idx_fold = subset_id.index("_fold")
            file_id = subset_id[(idx_fold + 5):]
            return int(file_id)
        except:
            return 0

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        idx = self._get_fold_index()

        urls = _URLS[_DATASETNAME]

        for key in urls:
            urls[key] = urls[key].format(fold_number=idx+1)

        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # Whatever you put in gen_kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": data_dir["train"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": data_dir["test"],
                    "split": "test",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": data_dir["validation"],
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        conll_dataset = load_conll_data(filepath)

        if self.config.schema == "source":
            for i, row in enumerate(conll_dataset):
                ex = {
                    "index": str(i),
                    "tokens": row["sentence"],
                    "tags": row["label"]
                }
                yield i, ex
        elif self.config.schema == "seacrowd_seq_label":
            for i, row in enumerate(conll_dataset):
                ex = {
                    "id": str(i),
                    "tokens": row["sentence"],
                    "labels": row["label"]
                }
                yield i, ex



"""
Knowledge Base Schema

This is a very general schema that covers many information extraction-style
tasks, including:

- Named entity recognition (NER)
- Named entity disambiguation/canonicalization/normalization (NED)
- Event extraction
- Relation extraction (RE)
- Coreference resolution

This schema assumes a document with child elements (e.g., entities, relations)
and a flat hierarchy of document passages.

"""
import datasets

features = datasets.Features(
    {
        "id": datasets.Value("string"),
        "passages": [{
            "id": datasets.Value("string"),
            "type": datasets.Value("string"),
            "text": datasets.Sequence(datasets.Value("string")),
            "offsets": datasets.Sequence([datasets.Value("int32")]),
        }],
        "entities": [{
            "id": datasets.Value("string"),
            "type": datasets.Value("string"),
            "text": datasets.Sequence(datasets.Value("string")),
            "offsets": datasets.Sequence([datasets.Value("int32")]),
            "normalized": datasets.Sequence({
                "db_name": datasets.Value("string"),
                "db_id": datasets.Value("string"),
            }),
        }],
        "events": [{
            "id": datasets.Value("string"),
            "type": datasets.Value("string"),
            # refers to the text_bound_annotation of the trigger
            "trigger": {
                "text": datasets.Sequence(datasets.Value("string")),
                "offsets": datasets.Sequence([datasets.Value("int32")]),
            },
            "arguments": datasets.Sequence({
                "role": datasets.Value("string"),
                "ref_id": datasets.Value("string"),
            }),
        }],
        "coreferences": [{
            "id": datasets.Value("string"),
            "entity_ids": datasets.Sequence(datasets.Value("string")),
        }],
        "relations": [{
            "id": datasets.Value("string"),
            "type": datasets.Value("string"),
            "arg1_id": datasets.Value("string"),
            "arg2_id": datasets.Value("string"),
            "normalized": datasets.Sequence({
                "db_name": datasets.Value("string"),
                "db_id": datasets.Value("string"),
            }),
        }],
    }
)



import json
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, Licenses

_CITATION = """\
@article{kautsar2023indotod,
    author={Kautsar, Muhammad Dehan Al and Nurdini, Rahmah Khoirussyifa' and Cahyawijaya, Samuel and Winata, Genta Indra and Purwarianti, Ayu},
    title={IndoToD: A Multi-Domain Indonesian Benchmark For End-to-End Task-Oriented Dialogue Systems},
    journal={arXiv preprint arXiv:2311.00958},
    year={2023},
}
"""

_LANGUAGES = ["ind"]
_LOCAL = False

_DATASETNAME = "indocamrest"

_DESCRIPTION = """\
IndoCamRest is a synthetic task-oriented dialogue system dataset that translated from Cambridge Restaurant 676 (CamRest) dataset (Wen et al., 2016) into the new Indonesian parallel dataset using the translation pipeline method including the delexicalization, translation, and delexicalization.
The dataset consists of 676 dialogues in the restaurant reservation domain, with a user and an agent talking to each other to search the restaurant near the user.
It also consists of slots and dialogue acts from the user and the agent.
"""

_HOMEPAGE = "https://github.com/dehanalkautsar/IndoToD/tree/main/IndoCamRest"

_LICENSE = Licenses.CC_BY_SA_4_0.value

_URLS = {
    _DATASETNAME: "https://raw.githubusercontent.com/dehanalkautsar/IndoToD/main/IndoCamRest/IndoCamRest676.json",
}

_SUPPORTED_TASKS = [Tasks.E2E_TASK_ORIENTED_DIALOGUE]

_SOURCE_VERSION = "1.0.0"

_SEACROWD_VERSION = "1.0.0"


class IndoCamRest(datasets.GeneratorBasedBuilder):
    """IndoToD: A Multi-Domain Indonesian Benchmark For End-to-End Task-Oriented Dialogue Systems"""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="indocamrest_source",
            version=SOURCE_VERSION,
            description="IndoToD: IndoCamRest source schema",
            schema="source",
            subset_id="indocamrest",
        ),
        SEACrowdConfig(
            name="indocamrest_seacrowd_tod",
            version=SEACROWD_VERSION,
            description="IndoToD: IndoCamRest SEACrowd End-to-end Task Oriented Dialogue schema",
            schema="seacrowd_tod",
            subset_id="indocamrest",
        ),
    ]

    DEFAULT_CONFIG_NAME = "indocamrest_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "index": datasets.Value("string"),
                    "dialogue_id": datasets.Value("int32"),
                    "finished": datasets.Value("string"),
                    "goal": {"constraints": [[datasets.Value("string")]], "request-slots": [datasets.Value("string")], "text": datasets.Value("string")},
                    "dial": [
                        {
                            "turn": datasets.Value("int32"),
                            "usr": {
                                "transcript": datasets.Value("string"),
                                "delex_transcript": datasets.Value("string"),
                                "slu": [{"act": datasets.Value("string"), "slots": [[datasets.Value("string")]]}],
                            },
                            "sys": {"sent": datasets.Value("string"), "delex_sent": datasets.Value("string"), "DA": [datasets.Value("string")]},
                        }
                    ],
                }
            )
        elif self.config.schema == "seacrowd_tod":
            features = schemas.tod_features
        else:
            raise NotImplementedError(f"Schema {self.config.schema} has not been implemented")

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        urls = _URLS[_DATASETNAME]
        data_dir = dl_manager.download_and_extract(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir,
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        with open(filepath, "r+") as fw:
            data = json.loads(fw.read())

        if self.config.schema == "source":
            for idx, example in enumerate(data):
                example["index"] = str(idx)
                yield str(idx), example

        elif self.config.schema == "seacrowd_tod":
            for idx, tod_dialogue in enumerate(data):
                example = {}
                example["dialogue_idx"] = idx

                dialogue = []
                for i in range(len(tod_dialogue["dial"]) + 1):
                    dial = {}
                    dial["turn_idx"] = i

                    # system_utterance properties
                    if i == 0:
                        # case if turn_idx == 0
                        dial["system_utterance"] = ""
                        dial["system_acts"] = []
                    else:
                        dial["system_utterance"] = tod_dialogue["dial"][i - 1]["sys"]["sent"]
                        # some system_acts is either to string or list of strings,
                        # converting all to list of strings
                        dial["system_acts"] = [[act] if isinstance(act, str) else act for act in tod_dialogue["dial"][i - 1]["sys"]["DA"]]

                    # user_utterance properties
                    dial["turn_label"] = []
                    dial["belief_state"] = []
                    if i == len(tod_dialogue["dial"]):
                        # case if turn_idx > len(dialogue) --> add dummy user_utterance
                        dial["user_utterance"] = ""
                    else:
                        dial["user_utterance"] = tod_dialogue["dial"][i]["usr"]["transcript"]
                        for j in range(len(tod_dialogue["dial"][i]["usr"]["slu"])):
                            dial["belief_state"].append({"slots": tod_dialogue["dial"][i]["usr"]["slu"][j]["slots"], "act": tod_dialogue["dial"][i]["usr"]["slu"][j]["act"]})

                    # append to dialogue
                    dialogue.append(dial)
                example["dialogue"] = dialogue
                yield str(idx), example



from pathlib import Path
from typing import Dict, List, Tuple
from urllib.parse import urljoin

import datasets
from pyarrow import parquet as pq

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks, Licenses

_CITATION = """\
@article{nguyen2023culturax,
    author    = {Thuat Nguyen and Chien Van Nguyen and Viet Dac Lai and Hieu Man and Nghia Trung Ngo and Franck Dernoncourt and Ryan A. Rossi and Thien Huu Nguyen},
    title     = {CulturaX: A Cleaned, Enormous, and Multilingual Dataset for Large Language Models in 167 Languages},
    journal   = {arXiv preprint arXiv:2309.09400},
    year      = {2023},
    url       = {https://arxiv.org/abs/2309.09400},
}
"""

_DATASETNAME = "culturax"
_DESCRIPTION = """\
CulturaX is a comprehensive multilingual dataset comprising 6.3 trillion tokens across 167
languages, designed for large language model development. It incorporates an advanced
cleaning and deduplication process, including language identification and fuzzy
deduplication with MinHash, to ensure high-quality data for model training. The dataset,
which spans 16TB in parquet format and 27TB when unpacked, is a combination of the latest
mC4 and OSCAR corpora, emphasizing non-English languages to support multilingual model
training. For data cleaning validation, CulturaX employs a SentencePiece tokenizer and
KenLM language models, utilizing recent Wikipedia dumps for perplexity scoring.
Before using this dataloader, please accept the acknowledgement at https://huggingface.co/datasets/uonlp/CulturaX and use huggingface-cli login for authentication.
"""

_LOCAL=False
_LANGUAGES = ["ind", "jav", "khm", "lao", "tgl", "min", "mya", "sun", "tha", "vie", "zlm", "ceb", "war", "cbk", "bcl"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)

_HOMEPAGE = "https://huggingface.co/datasets/uonlp/CulturaX"
_LICENSE = f"""{Licenses.OTHERS.value} | \
    The licence terms for CulturaX strictly follows those of mC4 and OSCAR. \
    Please refer to both below licenses when using this dataset. \
    - mC4 license: https://huggingface.co/datasets/allenai/c4#license \
    - OSCAR license: https://huggingface.co/datasets/oscar-corpus/OSCAR-2301#licensing-information \
"""
_BASE_URL = "https://huggingface.co/datasets/uonlp/CulturaX/resolve/main/{lang}/"

_SUPPORTED_TASKS = [Tasks.SELF_SUPERVISED_PRETRAINING]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class CulturaXDataset(datasets.GeneratorBasedBuilder):
    """CulturaX subset for SEA languages."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "ssp"
    SUBSETS = ["id", "jv", "km", "lo", "tl", "min", "my", "su", "th", "vi", "ms", "ceb", "war", "cbk", "bcl"]

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{subset}_source",
            version=datasets.Version(_SOURCE_VERSION),
            description=f"{_DATASETNAME} {subset} source schema",
            schema="source",
            subset_id=subset,
        )
        for subset in SUBSETS
    ] + [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_{subset}_seacrowd_ssp",
            version=datasets.Version(_SEACROWD_VERSION),
            description=f"{_DATASETNAME} {subset} SEACrowd schema",
            schema="seacrowd_ssp",
            subset_id=subset,
        )
        for subset in SUBSETS
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_jv_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "timestamp": datasets.Value("string"),
                    "url": datasets.Value("string"),
                    "source": datasets.Value("string"),
                }
            )
        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.ssp_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        base_path = _BASE_URL.format(lang=self.config.name.split("_")[1])

        checksum_url = urljoin(base_path, "checksum.sha256")
        checksum_path = Path(dl_manager.download(checksum_url))
        with open(checksum_path, encoding="utf-8") as f:
            filenames = [line.split()[1] for line in f if line]
            data_urls = [urljoin(base_path, filename) for filename in filenames]

        data_paths = list(map(Path, dl_manager.download([url for url in data_urls if url.endswith(".parquet")])))

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepaths": data_paths,
                    "split": "train",
                },
            )
        ]

    def _generate_examples(self, filepaths: [Path], split: str) -> Tuple[int, Dict]:
        """Yields examples as (key, example) tuples.

        Iterate over row groups in each filepaths, then yield each row as an example.
        """
        key = 0
        for filepath in filepaths:
            with open(filepath, "rb") as f:
                pf = pq.ParquetFile(f)
                for row_group in range(pf.num_row_groups):
                    df = pf.read_row_group(row_group).to_pandas()
                    for row in df.itertuples():
                        if self.config.schema == "source":
                            yield key, {
                                "text": row.text,
                                "timestamp": row.timestamp,
                                "url": row.url,
                                "source": row.source,
                            }
                        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
                            yield key, {
                                "id": str(key),
                                "text": row.text,
                            }
                        key += 1



# Taken from https://github.com/valentinakania/indocoref/blob/main/src/utils/file_utils.py
import os
import re
import pickle
import json
import logging
from pathlib import Path
from nltk.tokenize import sent_tokenize

logger = logging.getLogger(__name__)
logging.basicConfig(level = logging.INFO)

class FileUtils:
    @staticmethod
    def write_gold_cluster_to_json(output_dir, name, cluster):
        filename = Path(output_dir).joinpath("gold/" + name).with_suffix(".json")
        FileUtils.write_cluster_to_json(filename, cluster)

    @staticmethod
    def write_mps_result_to_json(output_dir, name, cluster):
        filename = Path(output_dir).joinpath("result-mps/" + name).with_suffix(".json")
        FileUtils.write_cluster_to_json(filename, cluster)

    @staticmethod
    def write_cluster_to_json(filename, cluster):
        os.makedirs(os.path.dirname(filename), exist_ok=True)
        data = {'type': 'clusters', 'clusters': cluster}
        with open(filename, 'w') as f:
            f.write(json.dumps(data))

    @staticmethod
    def write_to_pickle(temp_dir, name, objs):
        filename = Path(temp_dir).joinpath(name).with_suffix(".pkl")
        os.makedirs(os.path.dirname(filename), exist_ok=True)
        with open(filename, 'wb') as f:
            pickle.dump(objs, f)

    @staticmethod
    def read_pickle(temp_dir, name):
        filename = Path(temp_dir).joinpath(name)
        with open(filename, 'rb') as f:
            objs = pickle.load(f)
        return objs

    @staticmethod
    def read_annotated_file(annotated_dir, name):
        with open(Path(annotated_dir).joinpath(name), 'r', encoding='utf-8') as f:
            annotated = f.read()
        sentences = sent_tokenize(annotated)
        return (annotated, sentences)

    @staticmethod
    def read_passage_file(passage_dir, name):
        passage_name = re.sub(r"_[0-9]{8}-[0-9]{6}.+", ".txt", name)
        with open(Path(passage_dir).joinpath(passage_name), 'r', encoding='utf-8') as f:
            passage = f.read()
            sentences = sent_tokenize(passage)
        return sentences


import os
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks
from seacrowd.utils import schemas
import jsonlines
from nltk.tokenize.treebank import TreebankWordDetokenizer


_CITATION = """\
@article{aji2022paracotta,
  title={ParaCotta: Synthetic Multilingual Paraphrase Corpora from the Most Diverse Translation Sample Pair},
  author={Aji, Alham Fikri and Fatyanosa, Tirana Noor and Prasojo, Radityo Eko and Arthur, Philip and Fitriany, Suci and Qonitah, Salma and Zulfa, Nadhifa and Santoso, Tomi and Data, Mahendra},
  journal={arXiv preprint arXiv:2205.04651},
  year={2022}
}
"""

_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_LOCAL = False

_DATASETNAME = "paracotta_id"

_DESCRIPTION = """\
ParaCotta is a synthetic parallel paraphrase corpus across 17 languages: Arabic, Catalan, Czech, German, English, Spanish, Estonian, French, Hindi, Indonesian, Italian, Dutch, Ro- manian, Russian, Swedish, Vietnamese, and Chinese.
"""

_HOMEPAGE = "https://github.com/afaji/paracotta-paraphrase"

_LICENSE = "Unknown"

_URLS = {
    _DATASETNAME: "https://drive.google.com/uc?id=1QPyD4lOKxbXGUypA5ke6Y9_i9utq-QSQ",
}

_SUPPORTED_TASKS = [Tasks.PARAPHRASING]

# Dataset does not have versioning
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class ParaCotta(datasets.GeneratorBasedBuilder):
    """ParaCotta is a synthetic parallel paraphrase corpus across 17 languages: Arabic, Catalan, Czech, German, English, Spanish, Estonian, French, Hindi, Indonesian, Italian, Dutch, Ro- manian, Russian, Swedish, Vietnamese, and Chinese.
    """

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="paracotta_id_source",
            version=SOURCE_VERSION,
            description="paracotta_id source schema",
            schema="source",
            subset_id="paracotta_id",
        ),
        SEACrowdConfig(
            name="paracotta_id_seacrowd_t2t",
            version=SEACROWD_VERSION,
            description="paracotta_id Nusantara schema",
            schema="seacrowd_t2t",
            subset_id="paracotta_id",
        ),
    ]

    DEFAULT_CONFIG_NAME = "paracotta_id_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
          features = datasets.Features(
            {
              "id": datasets.Value("string"),
              "src": datasets.Value("string"),
              "tgt": datasets.Value("string"),
            }
          )
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        """Returns SplitGenerators."""
        urls = _URLS[_DATASETNAME]

        data_dir = Path(dl_manager.download(urls))
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir,
                    "split": "test",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        if self.config.schema == "source":
            with open(filepath, 'r') as f:
                data = f.readlines()
                id = 0
                for each_data in data:
                    each_data = each_data.strip('\n')
                    ex = {
                        "id": id,
                        "src": each_data.split('\t')[1],
                        "tgt": each_data.split('\t')[2],
                    }
                    id += 1
                    yield id, ex

        elif self.config.schema == "seacrowd_t2t":
            with open(filepath, 'r') as f:
                data = f.readlines()
                id = 0
                for each_data in data:
                    each_data = each_data.strip('\n')
                    ex = {
                        "id": id,
                        "text_1": each_data.split('\t')[1],
                        "text_2": each_data.split('\t')[2],
                        "text_1_name": "src",
                        "text_2_name": "tgt"
                    }
                    id += 1
                    yield id, ex
        else:
            raise ValueError(f"Invalid config: {self.config.name}")


# coding=utf-8
# Copyright 2022 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import json
from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks

_CITATION = """\
@misc{rizqullah2023qasina,
title={QASiNa: Religious Domain Question Answering using Sirah Nabawiyah},
author={Muhammad Razif Rizqullah and Ayu Purwarianti and Alham Fikri Aji},
year={2023},
eprint={2310.08102},
archivePrefix={arXiv},
primaryClass={cs.CL}
}
"""

_DATASETNAME = "qasina"

_DESCRIPTION = """\
Question Answering Sirah Nabawiyah Dataset (QASiNa) is Extractive \
QA Dataset which build to perform QA task in Sirah Nabawiyah domain.
"""

_HOMEPAGE = "https://github.com/rizquuula/QASiNa"

_LANGUAGES = ["ind"]

_LICENSE = Licenses.MIT.value
_LOCAL = False

_URLS = {
    _DATASETNAME: "https://github.com/rizquuula/QASiNa/raw/main/QASiNa.json",
}


_SUPPORTED_TASKS = [Tasks.QUESTION_ANSWERING]

_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


# TODO: Name the dataset class to match the script name using CamelCase instead of snake_case
class QasinaDataset(datasets.GeneratorBasedBuilder):
    """Question Answering Sirah Nabawiyah Dataset (QASiNa) is \
    Extractive QA Dataset which build to perform QA task in Sirah Nabawiyah domain."""

    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)

    SEACROWD_SCHEMA_NAME = "qa"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description=f"{_DATASETNAME} source schema",
            schema="source",
            subset_id=f"{_DATASETNAME}",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_{SEACROWD_SCHEMA_NAME}",
            version=SEACROWD_VERSION,
            description=f"{_DATASETNAME} SEACrowd schema",
            schema=f"seacrowd_{SEACROWD_SCHEMA_NAME}",
            subset_id=f"{_DATASETNAME}",
        ),
    ]

    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "context_id": datasets.Value("int32"),
                    "context": datasets.Value("string"),
                    "question_answers": datasets.Sequence({"type": datasets.Value("string"), "question": datasets.Value("string"), "answer": datasets.Value("string"), "answer_start": datasets.Value("int32"), "question_id": datasets.Value("int32")}),
                    "context_length": datasets.Value("int32"),
                    "context_title": datasets.Value("string"),
                }
            )

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            features = schemas.qa.features
            features["meta"] = {"context_title": datasets.Value("string"), "answer_start": datasets.Value("int32"),"context_length": datasets.Value("int32"), "type": datasets.Value("string")}

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        urls = _URLS[_DATASETNAME]
        filepath = dl_manager.download(urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": filepath,
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath: Path, split: str) -> Tuple[int, Dict]:
        with open(filepath) as file:
            dataset = json.load(file)

        if self.config.schema == "source":
            for i, line in enumerate(dataset):
                yield i, {
                    "context_id": line["context_id"],
                    "context": line["context"],
                    "question_answers": [
                        {
                            "type": subline["type"],
                            "question": subline["question"],
                            "answer": subline["answer"],
                            "answer_start": subline["answer_start"],
                            "question_id": subline["question_id"],
                        }
                        for subline in line["question_answers"]
                    ],
                    "context_length": line["context_length"],
                    "context_title": line["context_title"],
                }

        elif self.config.schema == f"seacrowd_{self.SEACROWD_SCHEMA_NAME}":
            for line in dataset:
                for question_answer in line["question_answers"]:
                    id = question_answer["question_id"]

                    yield id, {
                        "id": id,
                        "question_id": question_answer["question_id"],
                        "document_id": line["context_id"],
                        "question": question_answer["question"],
                        "type": "extractive",
                        "choices": [],
                        "context": line["context"],
                        "answer": [question_answer["answer"]],
                        "meta": {
                            "context_title": line["context_title"],
                            "answer_start": question_answer["answer_start"],
                            "context_length": line["context_length"],
                            "type": question_answer["type"],
                        },
                    }



import gzip
import json
from typing import List

from pathlib import Path
from typing import Dict, List, Tuple

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Licenses, Tasks


_DATASETNAME = "mc4_indo"
_DESCRIPTION = """\
    A thoroughly cleaned version of the Indonesia split of the multilingual colossal, cleaned version of Common Crawl's web crawl corpus (mC4). This portion represents the Indonesian language content that has been extracted and processed from the larger mC4 dataset. The extraction and cleaning process was conducted by AllenAI and resulted in a curated collection of Indonesian language data. For more information about the original mC4 dataset and its preparation, please refer to the source hosted at the address https://huggingface.co/datasets/allenai/c4.
"""

_HOMEPAGE = "https://huggingface.co/datasets/indonesian-nlp/mc4-id"
_LICENSE = Licenses.ODC_BY.value

_LANGUAGES = ["ind"]

_CITATION = """
@inproceedings{xue-etal-2021-mt5,
    title = "m{T}5: A Massively Multilingual Pre-trained Text-to-Text Transformer",
    author = "Xue, Linting  and
      Constant, Noah  and
      Roberts, Adam  and
      Kale, Mihir  and
      Al-Rfou, Rami  and
      Siddhant, Aditya  and
      Barua, Aditya  and
      Raffel, Colin",
    booktitle = "Proceedings of the 2021 Conference of the North American Chapter of the Association for Computational Linguistics: Human Language Technologies",
    month = jun,
    year = "2021",
    address = "Online",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2021.naacl-main.41",
    doi = "10.18653/v1/2021.naacl-main.41",
    pages = "483--498",
}
"""

_URLS = {"raw": "https://huggingface.co/datasets/munggok/mc4-id/resolve/main/mc4-id-filter/c4-id{split_suffix}.tfrecord-{index:05d}-of-{n_shards:05d}.json.gz"}

_CONFIGS = {"full": {"train": 1016, "validation": 8}}
# The entire dataset is 150 Gigs. You can adjust the number of "parquet" files you want to download here
# _CONFIGS = {
#     "full": {"train": 1, "validation": 1}
# }

_LOCAL = False

_SUPPORTED_TASKS = [Tasks.SELF_SUPERVISED_PRETRAINING]
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class MC4Indo(datasets.GeneratorBasedBuilder):
    SOURCE_VERSION = datasets.Version(_SOURCE_VERSION)
    SEACROWD_VERSION = datasets.Version(_SEACROWD_VERSION)
    DEFAULT_CONFIG_NAME = f"{_DATASETNAME}_source"

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name=f"{_DATASETNAME}_source",
            version=SOURCE_VERSION,
            description="mc4_indo source schema",
            schema="source",
            subset_id="mc4_indo",
        ),
        SEACrowdConfig(
            name=f"{_DATASETNAME}_seacrowd_ssp",
            version=SEACROWD_VERSION,
            description="mc4_indo SEACrowd schema",
            schema="seacrowd_ssp",
            subset_id="mc4_indo",
        ),
    ]

    def _info(self) -> datasets.DatasetInfo:
        if self.config.schema == "source":
            features = datasets.Features({"text": datasets.Value("string"), "timestamp": datasets.Value("string"), "url": datasets.Value("string")})

        elif self.config.schema == "seacrowd_ssp":
            features = schemas.self_supervised_pretraining.features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        data_urls = {}
        for split in ["train", "validation"]:
            data_urls[split] = [
                _URLS["raw"].format(
                    split_suffix="-validation" if split == "validation" else "",
                    index=index,
                    n_shards=8 if split == "validation" else 1024,
                )
                for index in range(_CONFIGS["full"][split])
            ]
        train_downloaded_files = dl_manager.download(data_urls["train"])
        validation_downloaded_files = dl_manager.download(data_urls["validation"])
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepaths": train_downloaded_files,
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepaths": validation_downloaded_files,
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepaths: [Path], split: str) -> Tuple[int, Dict]:
        id_ = 0
        for filepath in filepaths:
            with gzip.open(open(filepath, "rb"), "rt", encoding="utf-8") as f:
                for line in f:
                    if line:
                        example = json.loads(line)

                        if self.config.schema == "source":
                            yield id_, example
                        elif self.config.schema == "seacrowd_ssp":
                            seacrowd_json = {
                                "id": str(id_),
                                "text": str(example["text"]),
                            }
                            yield id_, seacrowd_json

                        id_ += 1



from pathlib import Path
from typing import List

import datasets

from seacrowd.utils import schemas
from seacrowd.utils.configs import SEACrowdConfig
from seacrowd.utils.constants import Tasks

_CITATION = """\
@inproceedings{guntara-etal-2020-benchmarking,
    title = "Benchmarking Multidomain {E}nglish-{I}ndonesian Machine Translation",
    author = "Guntara, Tri Wahyu  and
      Aji, Alham Fikri  and
      Prasojo, Radityo Eko",
    booktitle = "Proceedings of the 13th Workshop on Building and Using Comparable Corpora",
    month = may,
    year = "2020",
    address = "Marseille, France",
    publisher = "European Language Resources Association",
    url = "https://aclanthology.org/2020.bucc-1.6",
    pages = "35--43",
    language = "English",
    ISBN = "979-10-95546-42-9",
}
"""

_LOCAL = False
_LANGUAGES = ["ind"]  # We follow ISO639-3 language code (https://iso639-3.sil.org/code_tables/639/data)
_DATASETNAME = "indo_general_mt_en_id"
_DESCRIPTION = """\
"In the context of Machine Translation (MT) from-and-to English, Bahasa Indonesia has been considered a low-resource language,
and therefore applying Neural Machine Translation (NMT) which typically requires large training dataset proves to be problematic.
In this paper, we show otherwise by collecting large, publicly-available datasets from the Web, which we split into several domains: news, religion, general, and
conversation,to train and benchmark some variants of transformer-based NMT models across the domains.
We show using BLEU that our models perform well across them , outperform the baseline Statistical Machine Translation (SMT) models,
and perform comparably with Google Translate. Our datasets (with the standard split for training, validation, and testing), code, and models are available on https://github.com/gunnxx/indonesian-mt-data."
"""

_HOMEPAGE = "https://github.com/gunnxx/indonesian-mt-data"

_LICENSE = "Creative Commons Attribution Share-Alike 4.0 International"

_URLS = {
    _DATASETNAME: "https://github.com/gunnxx/indonesian-mt-data/archive/refs/heads/master.zip",
}

_SUPPORTED_TASKS = [Tasks.MACHINE_TRANSLATION]
# Dataset does not have versioning
_SOURCE_VERSION = "1.0.0"
_SEACROWD_VERSION = "1.0.0"


class IndoGeneralMTEnId(datasets.GeneratorBasedBuilder):
    """Indonesian General Domain MT En-Id is a machine translation dataset containing English-Indonesian parallel sentences collected from the general manuscripts."""

    BUILDER_CONFIGS = [
        SEACrowdConfig(
            name="indo_general_mt_en_id_source",
            version=datasets.Version(_SOURCE_VERSION),
            description="Indonesian General Domain MT En-Id source schema",
            schema="source",
            subset_id="indo_general_mt_en_id",
        ),
        SEACrowdConfig(
            name="indo_general_mt_en_id_seacrowd_t2t",
            version=datasets.Version(_SEACROWD_VERSION),
            description="Indonesian General Domain MT Nusantara schema",
            schema="seacrowd_t2t",
            subset_id="indo_general_mt_en_id",
        ),
    ]

    DEFAULT_CONFIG_NAME = "indo_general_mt_en_id_source"

    def _info(self):
        if self.config.schema == "source":
            features = datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "src": datasets.Value("string"),
                    "tgt": datasets.Value("string"),
                }
            )
        elif self.config.schema == "seacrowd_t2t":
            features = schemas.text2text_features

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager: datasets.DownloadManager) -> List[datasets.SplitGenerator]:
        urls = _URLS[_DATASETNAME]
        data_dir = Path(dl_manager.download_and_extract(urls)) / "indonesian-mt-data-master" / "general"

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": {
                        "en": [data_dir / "train.en.0", data_dir / "train.en.1", data_dir / "train.en.2", data_dir / "train.en.3"],
                        "id": [data_dir / "train.id.0", data_dir / "train.id.1", data_dir / "train.id.2", data_dir / "train.id.3"],
                    }
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": {
                        "en": [data_dir / "test.en"],
                        "id": [data_dir / "test.id"],
                    }
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": {
                        "en": [data_dir / "valid.en"],
                        "id": [data_dir / "valid.id"],
                    }
                },
            ),
        ]

    def _generate_examples(self, filepath: dict):

        data_en = None
        for file in filepath["en"]:
            if data_en is None:
                data_en = open(file, "r").readlines()
            else:
                data_en += open(file, "r").readlines()

        data_id = None
        for file in filepath["id"]:
            if data_id is None:
                data_id = open(file, "r").readlines()
            else:
                data_id += open(file, "r").readlines()

        data_en = list(map(str.strip, data_en))
        data_id = list(map(str.strip, data_id))

        if self.config.schema == "source":
            for id, (src, tgt) in enumerate(zip(data_en, data_id)):
                row = {
                    "id": str(id),
                    "src": src,
                    "tgt": tgt,
                }
                yield id, row
        elif self.config.schema == "seacrowd_t2t":
            for id, (src, tgt) in enumerate(zip(data_en, data_id)):
                row = {
                    "id": str(id),
                    "text_1": src,
                    "text_2": tgt,
                    "text_1_name": "eng",
                    "text_2_name": "ind",
                }
                yield id, row
        else:
            raise ValueError(f"Invalid config: {self.config.name}")
